//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2011). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for characteristic curve TOPO tunneled over ServiceChannel 
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define CC_HEADER       0   // active mode: parsing CC header
#define CC_DATA         1  // active mode: parsing CC data

#define CC_VERSION_LEN  2
#define CC_VERSION_MIN  200
#define CC_VERSION_HDRv3 300
#define CC_TYPE_LEN     2
#define CC_NUMBER_LEN   2
#define CC_KNR_LEN      22
#define CC_WKN_LEN      22
#define CC_NAME_LEN     22
#define CC_DATE_LEN     10
#define CC_COUNT_LEN    4
#define CC_BPV_LEN      4
#define CC_STARTX_LEN   4
#define CC_STEPX_LEN    4
#define CC_SETTING_LEN  4
#define CC_FW_VERSION_LEN  22  // Länge FW-Version
#define CC_FW_VERSION_FILL  2  // 2 Füll-Bytes wegen 4-Byte-Grenze

#define CC_RANGE_EXTENT_DIVIDER  10  // 1/10 --> 10% extra space around graph

#define CC_TO_TIME_VALUE  32000      // TIME_VALUE unit is 1/32 msec
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// assign header information (little endian)
#define LOAD_CC_VERSION(_x_, _i_)       var_CC_ ## _x_ ## _Version= var_SC_RSP_Buffer[## _i_ ##+1]*100 + var_SC_RSP_Buffer[## _i_ ##]; ## _i_ ## += CC_VERSION_LEN
#define LOAD_CC_TYPE(_x_, _i_)          var_CC_ ## _x_ ## _Type= NTOH(var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += CC_TYPE_LEN
#define LOAD_CC_NUMBER(_x_, _i_)        var_CC_ ## _x_ ## _Number= NTOH(var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += CC_NUMBER_LEN
#define LOAD_CC_KNR(_x_, _i_)           { int k; char sz[]; for (k= 0; k < CC_KNR_LEN; k++) { sz[k]= var_SC_RSP_Buffer[## _i_ ##+k]; } var_CC_ ## _x_ ## _KNR= sz;} ## _i_ ## += CC_KNR_LEN
#define LOAD_CC_WKN(_x_, _i_)           { int k; char sz[]; for (k= 0; k < CC_WKN_LEN; k++) { sz[k]= var_SC_RSP_Buffer[## _i_ ##+k]; } var_CC_ ## _x_ ## _WKN= sz;} ## _i_ ## += CC_WKN_LEN
#define LOAD_CC_NAME(_x_, _i_)          { int k; char sz[]; for (k= 0; k < CC_NAME_LEN; k++) { sz[k]= var_SC_RSP_Buffer[## _i_ ##+k]; } var_CC_ ## _x_ ## _Name= sz;} ## _i_ ## += CC_NAME_LEN
#define LOAD_CC_TIME_STORED(_x_, _i_)   { char sz[]; unsigned int v; v= var_SC_RSP_Buffer[##_i_##+7] + 2000; sz= itoa(v) + "-"; \
        v= var_SC_RSP_Buffer[##_i_##+6]; if(v<10) sz+="0"; sz += itoa(v) + "-"; \
        v= var_SC_RSP_Buffer[##_i_##+5]; if(v<10) sz+="0"; sz += itoa(v) + " "; \
        v= var_SC_RSP_Buffer[##_i_##+4]; if(v<10) sz+="0"; sz += itoa(v) + ":"; \
        v= var_SC_RSP_Buffer[##_i_##+3]; if(v<10) sz+="0"; sz += itoa(v) + ":"; \
        v= var_SC_RSP_Buffer[##_i_##+2]; if(v<10) sz+="0"; sz += itoa(v) + ","; \
        v= NTOH(var_SC_RSP_Buffer[##_i_##+1], var_SC_RSP_Buffer[##_i_##]); sz += itoa(v); \
        var_CC_ ##_x_## _time_stored= sz;} ##_i_## += CC_DATE_LEN
#define LOAD_CC_TIME_CREATED(_x_, _i_)   { char sz[]; unsigned int v; v= var_SC_RSP_Buffer[##_i_##+7] + 2000; sz= itoa(v) + "-"; \
        v= var_SC_RSP_Buffer[##_i_##+6]; if(v<10) sz+="0"; sz += itoa(v) + "-"; \
        v= var_SC_RSP_Buffer[##_i_##+5]; if(v<10) sz+="0"; sz += itoa(v) + " "; \
        v= var_SC_RSP_Buffer[##_i_##+4]; if(v<10) sz+="0"; sz += itoa(v) + ":"; \
        v= var_SC_RSP_Buffer[##_i_##+3]; if(v<10) sz+="0"; sz += itoa(v) + ":"; \
        v= var_SC_RSP_Buffer[##_i_##+2]; if(v<10) sz+="0"; sz += itoa(v) + ","; \
        v= NTOH(var_SC_RSP_Buffer[##_i_##+1], var_SC_RSP_Buffer[##_i_##]); sz += itoa(v); \
        var_CC_ ##_x_## _time_created= sz;} ##_i_## += CC_DATE_LEN

#define LOAD_CC_FW_VERSION(_x_, _i_)    { int k; char sz[];\
		if(var_CC_ ## _x_ ## _Version >= CC_VERSION_HDRv3) {\
		for (k= 0; k < CC_FW_VERSION_LEN; k++) { sz[k]= var_SC_RSP_Buffer[## _i_ ##+k]; } ## _i_ ## += CC_FW_VERSION_LEN+CC_FW_VERSION_FILL; } else { sz= "??.??"; }\
		var_CC_ ## _x_ ## _FwVersion= sz; }

#define LOAD_CC_NUMVALUES(_x_, _i_)     var_CC_ ## _x_ ## _NumValues= NTOHL (var_SC_RSP_Buffer[## _i_ ##+3], var_SC_RSP_Buffer[## _i_ ##+2], var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += CC_COUNT_LEN
#define LOAD_CC_BPV(_x_, _i_)           var_CC_ ## _x_ ## _BytesPerValue= NTOHL (var_SC_RSP_Buffer[## _i_ ##+3], var_SC_RSP_Buffer[## _i_ ##+2], var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += CC_BPV_LEN
#define LOAD_CC_STARTX(_x_, _i_)        lval= NTOHL (var_SC_RSP_Buffer[## _i_ ##+3], var_SC_RSP_Buffer[## _i_ ##+2], var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += CC_STARTX_LEN; var_CC_ ## _x_ ## _StartX= lval;
#define LOAD_CC_STEPX(_x_, _i_)         var_CC_ ## _x_ ## _StepX= NTOHL (var_SC_RSP_Buffer[## _i_ ##+3], var_SC_RSP_Buffer[## _i_ ##+2], var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += CC_STEPX_LEN
#define LOAD_CC_SETTING(_x_, _i_, _n_)  var_CC_ ## _x_ ## _Setting ## _n_ ##= NTOHL (var_SC_RSP_Buffer[## _i_ ##+3], var_SC_RSP_Buffer[## _i_ ##+2], var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += CC_SETTING_LEN

// assign data information (little endian)
#define LOAD_CC_DATA1(_x_, _i_, _k_)  arr_CC_ ## _x_ ## [ ## _k_ ## ] = var_SC_RSP_Buffer[## _i_ ##]; ## _i_ ## += 1
#define LOAD_CC_DATA2(_x_, _i_, _k_)  arr_CC_ ## _x_ ## [ ## _k_ ## ] = NTOH (var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += 2
#define LOAD_CC_DATA4(_x_, _i_, _k_)  arr_CC_ ## _x_ ## [ ## _k_ ## ] = NTOHL (var_SC_RSP_Buffer[## _i_ ##+3], var_SC_RSP_Buffer[## _i_ ##+2], var_SC_RSP_Buffer[## _i_ ##+1], var_SC_RSP_Buffer[## _i_ ##]); ## _i_ ## += 4

// calculate relative value	(var_CC_XXXX_Setting2 is reference value <=> 100%)
#define CALCULATE_CC_RELVAL(_x_, _k_)    if (var_CC_ ## _x_ ## _Setting2 != 0) { arr_CC_ ## _x_ ## [ ## _k_ ## ] = (arr_CC_ ## _x_ ## [ ## _k_ ## ] * 100.0 )/var_CC_ ## _x_ ## _Setting2 + .5; }

// check for and ignore special values (replace with predecessor or 0)
#define CHECK_CC_IGNOREDVAL (_x_, _k_, _v_)  if (arr_CC_ ## _x_ ## [## _k_ ##] == ## _v_ ##) { \
    if (## _k_ ## > 0) arr_CC_ ## _x_ ## [## _k_ ##]= arr_CC_ ## _x_ ## [## _k_ ##-1]; else arr_CC_ ## _x_ ## [## _k_ ##]= 0; \
	}

// check correct version (--> exit at failure!) 
#define CHECK_CC_VERSION(_x_)    if (var_CC_ ## _x_ ## _Version < CC_VERSION_MIN){\
    _TRACE("### --> read_CC_ ## _x_ ## (): wrong file version (version= " + itoa(var_CC_ ## _x_ ## _Version) + ") --> exit");\
	PUT_MESSAGE("Wrong data file version --> Exit!");\
	return (SC_ERROR); }

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TEMPLATE VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE for active mode in CC reading = {CC_HEADER, CC_DATA}
// 
VARIABLE var_CC_mode
{
	LABEL		"CC mode data";
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(1);
	HANDLING		READ;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE for field index of next data to read
// 
VARIABLE var_CC_DataIndex
{
	LABEL		"Index of data";  // [TXT_var_CC_DataIndex];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(4);
	HANDLING		READ;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE for name of recent processed CC
// 
VARIABLE var_CC_ActiveTask
{
	LABEL		"CC_ActiveTask";  // [TXT_var_CC_ActiveTask];
	CLASS		LOCAL;
	TYPE		ASCII(22);
	HANDLING		READ;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE for message in progress window
// 
VARIABLE var_CC_ActiveMessage
{
	LABEL		"CC_ActiveMessage";  // [TXT_var_CC_ActiveMessage];
	CLASS		LOCAL;
	TYPE		ASCII(101);
	HANDLING		READ;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE for progress bar/status window
// 
VARIABLE var_CC_Progress
{
	LABEL		"CC_Progress";  // [TXT_var_CC_Progress];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER (1)
	{
	    DEFAULT_VALUE    0;
	    MIN_VALUE    0;
	    MAX_VALUE    100;
	}
	CONSTANT_UNIT    "%";
	HANDLING		READ;
}


//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC header
// 
VARIABLE var_CC_Version
{
	LABEL		"Version|de|Version"; // [TXT_var_CC_Version];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(2)
	{
	    DEFAULT_VALUE     0;
        SCALING_FACTOR    0.01;
        EDIT_FORMAT       ".2f";
        DISPLAY_FORMAT    ".2f";
	}
	HANDLING		READ;
	VALIDITY		TRUE;
}

VARIABLE var_CC_Type
{
	LABEL		"Type|de|Typ";  // [TXT_var_CC_Type];
	CLASS		LOCAL;
	TYPE		ENUMERATED (2)
	{
		{ 0, "Curve torque over position OPEN|de|Drehmoment-Position-Kennlinie (AUF)" /*[TXT_CC_TYPE_TOPOO]*/ },
		{ 1, "Curve torque over position CLOSE|de|Drehmoment-Position-Kennlinie (ZU)" /*[TXT_CC_TYPE_TOPOC]*/ },
		{ 2, "Histogram runtime over position|de|Motorlaufzeit-Position-Histogramm" /*[TXT_CC_TYPE_RTPO]*/ },
		{ 3, "Histogram runtime over temperature|de|Motorlaufzeit-Temperatur-Histogramm" /*[TXT_CC_TYPE_RTTE]*/ },
		{ 4, "Curve temperature over time|de|Temperatur-Zeit-Kennlinie" /*[TXT_CC_TYPE_TETI]*/ },
		{ 5, "Histogram vibration over frequency|de|Vibration-Frequenz-Histogramm" /*[TXT_CC_TYPE_AMFR]*/ },
		{ 6, "Curve humidity over time|de|Feuchte-Zeit-Kennlinie" /*[TXT_CC_TYPE_HUTI]*/ },
		{ 7, "Curve position over time|de|Position-Zeit-Kennlinie" /*[TXT_CC_TYPE_POTI]*/ },
		{ 8, "Histogram runtime over torque|de|Motorlaufzeit-Drehmoment-Histogramm" /*[TXT_CC_TYPE_RTTO]*/ },
		{ 9, "Curve pressure over time|de|Druck-Zeit-Kennlinie" /*[TXT_CC_TYPE_PRTI]*/ },
		{10, "Curve torque over time|de|Drehmoment-Zeit-Kennlinie" /*[TXT_CC_TYPE_TOTI]*/ },
	}
	HANDLING		READ;
}

VARIABLE var_CC_Number
{
	LABEL		"Number|de|Nummer";  // [TXT_var_CC_Number];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER (2);
	HANDLING		READ;
}

VARIABLE var_CC_KNR
{
	LABEL		"KNR|de|KNR";  // [TXT_var_CC_KNR];
	CLASS		LOCAL;
	TYPE		ASCII(CC_KNR_LEN);
	HANDLING		READ;
}

VARIABLE var_CC_WKN
{
	LABEL		"WKN|de|WKN";  // [TXT_var_CC_WKN];
	CLASS		LOCAL;
	TYPE		ASCII(CC_WKN_LEN);
	HANDLING		READ;
}

VARIABLE var_CC_Name
{
	LABEL		"Name|de|Name";  // [TXT_var_CC_Name];
	CLASS		LOCAL;
	TYPE		ASCII(CC_NAME_LEN);
	HANDLING		READ;
}

VARIABLE var_CC_FwVersion
{
	LABEL		"Firmware version|de|Firmware-Version";  // [TXT_var_CC_FwVersion];
	CLASS		LOCAL;
	TYPE		ASCII(CC_FW_VERSION_LEN);
	HANDLING		READ;
}

VARIABLE var_CC_time_stored
{
	LABEL		"Time (stored)|de|Zeit (gespeichert)";  // [TXT_var_time_stored];
	CLASS		LOCAL;
	TYPE		ASCII(24);
	HANDLING		READ;
}

VARIABLE var_CC_time_created
{
	LABEL		"Time (created)|de|Zeit (erstellt)";  // [TXT_var_time_created];
	CLASS		LOCAL;
	TYPE		ASCII(24);
	HANDLING		READ;
}

VARIABLE var_CC_NumValues
{
	LABEL		"Number of values|de|Anzahl Werte";  // [TXT_var_CC_NumValues];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(4);
	HANDLING		READ;
}

VARIABLE var_CC_BytesPerValue
{
	LABEL		"Bytes per value|de|Bytes pro Wert";  // [TXT_var_CC_BytesPerValue];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(4);
	HANDLING		READ;
}

VARIABLE var_CC_StartX
{
	LABEL		"Start x-axis|de|Start x-Achse";  // [TXT_var_CC_StartX];
	CLASS		LOCAL;
	TYPE		FLOAT;
	HANDLING		READ;
}

VARIABLE var_CC_StepX
{
	LABEL		"Step x-axis|de|Schrittweite x-Achse";  // [TXT_var_CC_StepX];
	CLASS		LOCAL;
	TYPE		FLOAT;
	HANDLING		READ;
}

VARIABLE var_CC_Setting1
{
	LABEL		"Setting 1|de|Einstellung 1";  // [TXT_var_CC_Setting1];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(4);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

VARIABLE var_CC_Setting2
{
	LABEL		"Setting 2|de|Einstellung 2";  // [TXT_var_CC_Setting2];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(4);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

VARIABLE var_CC_Setting3
{
	LABEL		"Setting 3|de|Einstellung 3";  // [TXT_var_CC_Setting3];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(4);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

VARIABLE var_CC_Setting4
{
	LABEL		"Setting 4|de|Einstellung 4";  // [TXT_var_CC_Setting4];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(4);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

VARIABLE var_CC_MinVal
{
	LABEL		"Minimal value|de|Minimalwert";  // [TXT_var_CC_MinVal];
	CLASS		LOCAL;
	TYPE		INTEGER(4);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

VARIABLE var_CC_MaxVal
{
	LABEL		"Maximal value|de|Maximalwert";  // [TXT_var_CC_MaxVal];
	CLASS		LOCAL;
	TYPE		INTEGER(4);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

VARIABLE var_CC_data_int8
{
	LABEL		"Data (int8)";  // [TXT_var_CC_data_int8];
	CLASS		LOCAL;
	TYPE		INTEGER(1);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

VARIABLE var_CC_data_int16
{
	LABEL		"Data (int16)";  // [TXT_var_CC_data_int16];
	CLASS		LOCAL;
	TYPE		INTEGER(2);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

VARIABLE var_CC_data_int32
{
	LABEL		"Data (int32)";  // [TXT_var_CC_data_int32];
	CLASS		LOCAL;
	TYPE		INTEGER(4);
	HANDLING		READ;
	CONSTANT_UNIT    " ";
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// COMMON VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
var_CC_MotorRuntime LIKE VARIABLE PRM_163
{
	REDEFINE CLASS	LOCAL;
#ifdef _V910_	
	REDEFINE TYPE	TIME_VALUE(4);
#else	
	REDEFINE TYPE	TIME_VALUE;
#endif	
	REDEFINE CONSTANT_UNIT	"hh:mm:ss";
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "AC2_SC.inc"
#include "AC2_CCTOPO.inc"
#include "AC2_CCTETI.inc"
// #include "AC2_CCPOTI.inc" (2013-07-12, WM-PM: preliminary to deactivate)
#include "AC2_CCRTPO.inc"
#include "AC2_CCRTTO.inc"
#include "AC2_CCRTTE.inc"
#include "AC2_CCAMFR.inc"


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MENUS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU  menu_CC_empty (only place holder for incomplete menus)
// 

MENU  menu_CC_empty
{
    LABEL    "";
	STYLE    WINDOW;
	ACCESS   OFFLINE;
	ITEMS
	{
		" "
	}
	VALIDITY    FALSE;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU  menu_CC_main -- root menu for characteristic curves
// 

MENU  menu_CC_main
{
    LABEL    "Characteristic curves|de|Kennlinien";  // [TXT_menu_CC_main_window];
	STYLE    MENU;
	ACCESS   OFFLINE;
	ITEMS
	{
#if defined (CC_TOPO) && (!defined (_V811_) || defined (_V820_)) // in PDM v811 connection is to slow
        menu_CC_TOPO,
#endif
#if defined (CC_TETI) && (!defined (_V811_) || defined (_V820_)) // in PDM v811 connection is to slow
        menu_CC_TETI,
#endif
#if defined (CC_POTI) && (!defined (_V811_) || defined (_V820_)) // in PDM v811 connection is to slow
        menu_CC_POTI,
#endif
#if defined (CC_RTPO)
        menu_CC_RTPO,
#endif
#if defined (CC_RTTO)
        menu_CC_RTTO,
#endif
#if defined (CC_RTTE)
        menu_CC_RTTE,
#endif
#if defined (CC_AMFR)
        menu_CC_AMFR,
#endif
        menu_CC_empty
	}
	
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU  menu_CC_progress_window (only a sketch/trial)
// 

MENU  menu_CC_progress_window
{
    LABEL    "Progress|de|Fortschritt";  // [TXT_menu_CC_progress_window];
	STYLE    WINDOW;
	ACCESS   OFFLINE;
	ITEMS
	{
	    var_CC_ActiveTask (NO_LABEL),
		chart_CC_Progress,
		var_CC_ActiveMessage (NO_LABEL)
	}
}

CHART  chart_CC_Progress
{
    LABEL    "";
	TYPE    HORIZONTAL_BAR;
	MEMBERS
	{
	    SOURCE, source_CC_Progress;
	}
}

SOURCE source_CC_Progress
{
    LABEL    "source_CC_Progress";
	MEMBERS
	{
	    VALUE1, var_CC_Progress;
	}	
}

/* --------------------------------------------------------------------------------------------------------------------
 *   End of file
 * --------------------------------------------------------------------------------------------------------------------
 */
