//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2011). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for characteristic curve AMFR tunneled over ServiceChannel 
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define CC_AMFR     1
#define CC_AMFR_DATA_COUNT	10
#define CC_AMFR_SKIP        1   // we take every sample
#define CC_AMFR_ARRAY_SIZE	(CC_AMFR_DATA_COUNT/CC_AMFR_SKIP)
#define CC_AMFR_XUNIT       "Hz"
#define CC_AMFR_XMIN        0
#define CC_AMFR_XMAX        200
#define CC_AMFR_XSTEP       20
#define CC_AMFR_YUNIT       "%"
#define CC_AMFR01   1
#define CC_AMFR02   2
#define CC_AMFR03   3

#define CC_AMFR01_FILE   "CCAMFR01.BIN"
#define CC_AMFR02_FILE   "CCAMFR02.BIN" /* not loaded */
#define CC_AMFR03_FILE   "CCAMFR03.BIN" /* not loaded */

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VARIABLE var_CC_AMFR_Selector
{
    LABEL    "Curve Selection|de|Kennlinienauswahl";  // [TXT_var_CC_Selector];
	CLASS    LOCAL;
	TYPE     ENUMERATED (2)
	{
        DEFAULT_VALUE	1;
        { CC_AMFR01, "Vibration over frequency (X)|de|Vibration-Frequenz (X)"/*[TXT_wave_CC_AMFR01]*/ },
        // disabled { CC_AMFR02, "Curve vibration frequency (Y)|de|Kurve Vibration-Frequenz (Y)"/*[TXT_wave_CC_AMFR02]*/ },
        // disabled { CC_AMFR03, "Curve vibration frequency (Z)|de|Kurve Vibration-Frequenz (Z)"/*[TXT_wave_CC_AMFR03]*/ },
	}
}

var_CC_AMFR_Value LIKE VARIABLE var_CC_data_int32{REDEFINE CONSTANT_UNIT CC_AMFR_YUNIT;}
var_CC_AMFR_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_AMFR_YUNIT;}
var_CC_AMFR_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_AMFR_YUNIT;}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC AMFR01
// 
var_CC_AMFR01_Version LIKE VARIABLE var_CC_Version{}
var_CC_AMFR01_Type LIKE VARIABLE var_CC_Type{}
var_CC_AMFR01_Number LIKE VARIABLE var_CC_Number{}
var_CC_AMFR01_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_AMFR01_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_AMFR01_Name LIKE VARIABLE var_CC_Name{}
var_CC_AMFR01_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_AMFR01_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_AMFR01_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_AMFR01_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_AMFR01_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_AMFR_XMIN;}
var_CC_AMFR01_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_AMFR_XSTEP;}
var_CC_AMFR01_Setting1 LIKE VARIABLE var_CC_Setting1{}
var_CC_AMFR01_Setting2 LIKE VARIABLE var_CC_Setting2{}
var_CC_AMFR01_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_AMFR01_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_AMFR01_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_AMFR01_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_AMFR01_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_AMFR_YUNIT;}
var_CC_AMFR01_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_AMFR_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC AMFR01 values
// 
ARRAY arr_CC_AMFR01
{
    LABEL    "CC AMFR01 values";  // [TXT_arr_CC_AMFR01];
	NUMBER_OF_ELEMENTS    CC_AMFR_ARRAY_SIZE;
	TYPE    var_CC_AMFR_Value;
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GRAPHS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

GRAPH graph_CC_AMFR
{
	LABEL 	"Histogram vibration over frequency|de|Vibration-Frequenz-Histogramm"; // [TXT_graph_CC_AMFR01];
	X_AXIS 	xaxis_CC_AMFR;
	WIDTH	X_SMALL;
	MEMBERS
	{
		WAVEFORM1, wave_CC_AMFR01;
	}
}

AXIS  	xaxis_CC_AMFR
{
    LABEL            "Frequency|de|Frequenz";
    CONSTANT_UNIT    CC_AMFR_XUNIT;
	MIN_VALUE        CC_AMFR_XMIN;
	MAX_VALUE        CC_AMFR_XMAX;
}

AXIS  	yaxis_CC_AMFR
{
    LABEL  "Vibration|de|Vibration";
	// no CONSTANT_UNIT -- UNIT given by MIN_/MAX_VALUE
	MIN_VALUE    var_CC_AMFR_MinVal;
	MAX_VALUE    var_CC_AMFR_MaxVal;
}

WAVEFORM  wave_CC_AMFR01
{
	LABEL   "Vibration over frequency (X)|de|Vibration-Frequenz (X)"; // [TXT_wave_CC_AMFR01];
	Y_AXIS 	yaxis_CC_AMFR;
	TYPE    YT
	{
	    X_INITIAL    var_CC_AMFR01_StartX;
		X_INCREMENT  var_CC_AMFR01_StepX;
		NUMBER_OF_POINTS    CC_AMFR_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_AMFR01
		}
	}
	EMPHASIS  IF (var_CC_AMFR_Selector == CC_AMFR01) {TRUE;} ELSE {FALSE;}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MENUES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU menu_CC_AMFR main page of CC AMFR
// 
MENU menu_CC_AMFR
{
	LABEL    "Histogram vibration over frequency|de|Vibration-Frequenz-Histogramm";  // [TXT_menu_CC_AMFR];
	STYLE    WINDOW;
	ACCESS   OFFLINE;
	ITEMS
	{
	    load_CC_AMFR01,
		COLUMNBREAK,
	    "\t",  // placeholder
		ROWBREAK,
		graph_CC_AMFR,
		COLUMNBREAK,
		var_CC_AMFR_Selector (NO_LABEL),
		group_CC_AMFR
	}
	PRE_EDIT_ACTIONS
	{
	    calc_CC_AMFR()
	}
}


//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU group_CC_AMFR  - properties of selected CC AMFR
// 
MENU group_CC_AMFR
{
	LABEL    "";
	STYLE    GROUP;
	ACCESS   OFFLINE;
	ITEMS
	{
	    SELECT (var_CC_AMFR_Selector)
		{
			DEFAULT: var_CC_AMFR01_WKN, var_CC_AMFR01_KNR, var_CC_AMFR01_time_created, var_CC_AMFR01_time_stored, var_CC_AMFR01_MinVal, var_CC_AMFR01_MaxVal, var_CC_AMFR01_FwVersion
			CASE CC_AMFR01: var_CC_AMFR01_WKN, var_CC_AMFR01_KNR, var_CC_AMFR01_time_created, var_CC_AMFR01_time_stored, var_CC_AMFR01_MinVal, var_CC_AMFR01_MaxVal, var_CC_AMFR01_FwVersion
		}
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_AMFR  - calculate auxiliary values for graph CC AMFR
// 
METHOD calc_CC_AMFR
{
    LABEL    "calc_CC_AMFR"; // [TXT_calc_CC_AMFR];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    int extent;
	    // calculate properties of waveforms
	    calc_CC_AMFR01();

        // MinValue of graph
        var_CC_AMFR_MinVal= 65535;
		if (var_CC_AMFR01_MinVal < var_CC_AMFR_MinVal) var_CC_AMFR_MinVal= var_CC_AMFR01_MinVal;
        // MaxValue of graph
        var_CC_AMFR_MaxVal= -65536;
		if (var_CC_AMFR01_MaxVal > var_CC_AMFR_MaxVal) var_CC_AMFR_MaxVal= var_CC_AMFR01_MaxVal;

		// adjust Min-/MaxVal for better display
		extent= (var_CC_AMFR_MaxVal - var_CC_AMFR_MinVal)/CC_RANGE_EXTENT_DIVIDER;
		var_CC_AMFR_MaxVal += extent;
		var_CC_AMFR_MinVal -= extent;
		
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_AMFR01  - calculate auxiliary values for CC AMFR01
// 
METHOD calc_CC_AMFR01
{
    LABEL    "calc_CC_AMFR01"; // [TXT_calc_CC_AMFR01];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_AMFR01_MinVal= 65535;
        var_CC_AMFR01_MaxVal= -65536;
		
		for (i= 0; i < CC_AMFR_ARRAY_SIZE; i++)
		{
		    if (arr_CC_AMFR01[i] < var_CC_AMFR01_MinVal) var_CC_AMFR01_MinVal= arr_CC_AMFR01[i];
		    if (arr_CC_AMFR01[i] > var_CC_AMFR01_MaxVal) var_CC_AMFR01_MaxVal= arr_CC_AMFR01[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_AMFR01  - load CC file CCAMFR01.BIN
// 
METHOD load_CC_AMFR01
{
    LABEL    "Load histogram vibration over frequency|de|Laden Vibration-Frequenz-Histogramm"; // [TXT_load_CC_AMFR01];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_AMFR01
			sz= "Read file " CC_AMFR01_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_AMFR01();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_AMFR();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_AMFR01  -  read and parse CC file
// 
METHOD read_CC_AMFR01
{
    LABEL    "Read file CCAMFR01.BIN|de|Lesen Datei CCAMFR01.BIN"; // [TXT_read_CC_AMFR01];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCAMFR01";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_AMFR01(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_AMFR01_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_AMFR01(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_AMFR01(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_AMFR01(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(AMFR01, index);
					    LOAD_CC_TYPE(AMFR01, index);
						LOAD_CC_NUMBER(AMFR01, index);
						LOAD_CC_KNR(AMFR01, index);
						LOAD_CC_WKN(AMFR01, index);
						LOAD_CC_NAME(AMFR01, index);
						LOAD_CC_TIME_STORED(AMFR01, index);
						LOAD_CC_TIME_CREATED(AMFR01, index);
						LOAD_CC_NUMVALUES(AMFR01, index);
						LOAD_CC_BPV(AMFR01, index);
						LOAD_CC_STARTX(AMFR01, index);
						LOAD_CC_STEPX(AMFR01, index);
						LOAD_CC_SETTING(AMFR01, index, 1);
						LOAD_CC_SETTING(AMFR01, index, 2);
						LOAD_CC_SETTING(AMFR01, index, 3);
						LOAD_CC_SETTING(AMFR01, index, 4);
						// load FW-Version (only from CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(AMFR01, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(AMFR01);
						// adjust x-axis properties
						xaxis_CC_AMFR.MIN_VALUE= var_CC_AMFR01_StartX;
						xaxis_CC_AMFR.MAX_VALUE= var_CC_AMFR01_StartX + var_CC_AMFR01_NumValues * CC_AMFR_XSTEP;
						var_CC_AMFR01_StartX += CC_AMFR_XSTEP/2;
						var_CC_AMFR01_StepX= CC_AMFR_XSTEP;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_AMFR01_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(AMFR01, index, n);
									CALCULATE_CC_RELVAL(AMFR01, n);
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(AMFR01, index, n);
									CALCULATE_CC_RELVAL(AMFR01, n);
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(AMFR01, index, n);
									CALCULATE_CC_RELVAL(AMFR01, n);
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_AMFR01_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_AMFR01_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_AMFR01(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
