//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2011). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for characteristic curve POTI tunneled over ServiceChannel 
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define CC_POTI     1
#define CC_POTI_DATA_COUNT	10000
#define CC_POTI_SKIP        10  // every 10th 
#define CC_POTI_ARRAY_SIZE	(CC_POTI_DATA_COUNT/CC_POTI_SKIP)
#define CC_POTI_XUNIT        "h"
#define CC_POTI_YUNIT        "%"
#define CC_POTI_IGNOREDVAL   9999

#define CC_POTI01   1
#define CC_POTI02   2

#define CC_POTI01_FILE   "CCPOTI01.BIN"
#define CC_POTI02_FILE   "CCPOTI02.BIN"

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VARIABLE var_CC_POTI_Selector
{
    LABEL    "Curve Selection|de|Kennlinienauswahl";  // [TXT_var_CC_Selector];
	CLASS    LOCAL;
	TYPE     ENUMERATED (2)
	{
        DEFAULT_VALUE	1;
        { CC_POTI01, "Position time over time (actual value)|de|Position-Zeit (Istwert)"/*[TXT_wave_CC_POTI01]*/ },
        { CC_POTI02, "Position time over time (set value)|de|Position-Zeit (Sollwert)"/*[TXT_wave_CC_POTI02]*/ },
	}
}

var_CC_POTI_Value LIKE VARIABLE var_CC_data_int8{REDEFINE CONSTANT_UNIT CC_POTI_YUNIT;}
var_CC_POTI_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_POTI_YUNIT;}
var_CC_POTI_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_POTI_YUNIT;}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC POTI01
// 
var_CC_POTI01_Version LIKE VARIABLE var_CC_Version{}
var_CC_POTI01_Type LIKE VARIABLE var_CC_Type{}
var_CC_POTI01_Number LIKE VARIABLE var_CC_Number{}
var_CC_POTI01_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_POTI01_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_POTI01_Name LIKE VARIABLE var_CC_Name{}
var_CC_POTI01_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_POTI01_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_POTI01_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_POTI01_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_POTI01_StartX LIKE VARIABLE var_CC_StartX{}
var_CC_POTI01_StepX LIKE VARIABLE var_CC_StepX{}
var_CC_POTI01_Setting1 LIKE VARIABLE var_CC_Setting1{}
var_CC_POTI01_Setting2 LIKE VARIABLE var_CC_Setting2{}
var_CC_POTI01_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_POTI01_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_POTI01_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_POTI01_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_POTI01_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_POTI_YUNIT;}
var_CC_POTI01_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_POTI_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC POTI01 values
// 
ARRAY arr_CC_POTI01
{
    LABEL    "CC POTI01 values";  // [TXT_arr_CC_POTI01];
	NUMBER_OF_ELEMENTS    CC_POTI_ARRAY_SIZE;
	TYPE    var_CC_POTI_Value;
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC POTI01 breaks
// 
ARRAY arr_CC_POTI01_breaks
{
    LABEL    "CC POTI01 breaks";  // [TXT_arr_CC_POTI01_breaks];
	NUMBER_OF_ELEMENTS    CC_POTI_ARRAY_SIZE;
	TYPE    var_CC_StepX;
}
VARIABLE var_CC_POTI01_breaksindex
{
	LABEL		"CC POTI01 breaks index";  // [TXT_var_CC_POTI01_breaksindex];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(2)
	{
	    DEFAULT_VALUE   0;
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC POTI02
// 
var_CC_POTI02_Version LIKE VARIABLE var_CC_Version{}
var_CC_POTI02_Type LIKE VARIABLE var_CC_Type{}
var_CC_POTI02_Number LIKE VARIABLE var_CC_Number{}
var_CC_POTI02_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_POTI02_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_POTI02_Name LIKE VARIABLE var_CC_Name{}
var_CC_POTI02_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_POTI02_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_POTI02_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_POTI02_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_POTI02_StartX LIKE VARIABLE var_CC_StartX{}
var_CC_POTI02_StepX LIKE VARIABLE var_CC_StepX{}
var_CC_POTI02_Setting1 LIKE VARIABLE var_CC_Setting1{}
var_CC_POTI02_Setting2 LIKE VARIABLE var_CC_Setting2{}
var_CC_POTI02_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_POTI02_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_POTI02_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_POTI02_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_POTI02_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_POTI_YUNIT;}
var_CC_POTI02_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_POTI_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC POTI02 values
// 
ARRAY arr_CC_POTI02
{
    LABEL    "CC POTI02 values";  // [TXT_arr_CC_POTI02];
	NUMBER_OF_ELEMENTS    CC_POTI_ARRAY_SIZE;
	TYPE    var_CC_POTI_Value;
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC POTI02 breaks
// 
ARRAY arr_CC_POTI02_breaks
{
    LABEL    "CC POTI02 breaks";  // [TXT_arr_CC_POTI02_breaks];
	NUMBER_OF_ELEMENTS    CC_POTI_ARRAY_SIZE;
	TYPE    var_CC_StepX;
}
VARIABLE var_CC_POTI02_breaksindex
{
	LABEL		"CC POTI02 breaks index";  // [TXT_var_CC_POTI02_breaksindex];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(2)
	{
	    DEFAULT_VALUE   0;
	}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GRAPHS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

GRAPH graph_CC_POTI
{
	LABEL 	"Characteristic curves position over time|de|Position-Zeit-Kennlinien"; // [TXT_graph_CC_POTI];
	X_AXIS 	xaxis_CC_POTI;
	WIDTH	X_SMALL;
	MEMBERS
	{
		WAVEFORM1, wave_CC_POTI01;
		IF (var_CC_POTI01_breaksindex > 0) { WAVEFORM2, wave_CC_POTI01_breaks; }
		WAVEFORM3, wave_CC_POTI02;
		IF (var_CC_POTI02_breaksindex > 0) { WAVEFORM4, wave_CC_POTI02_breaks; }
	}
}

AXIS  	xaxis_CC_POTI
{
    LABEL  "Time before|de|Zeit zurck"; // [TXT_xaxis_CC_POTI];
	CONSTANT_UNIT    CC_POTI_XUNIT;
	MIN_VALUE    var_CC_POTI01_StartX;
	MAX_VALUE    0;
}

AXIS  	yaxis_CC_POTI
{
    LABEL  "Position|de|Position";
	// no CONSTANT_UNIT -- UNIT given by MIN_/MAX_VALUE
	MIN_VALUE    var_CC_POTI_MinVal;
	MAX_VALUE    var_CC_POTI_MaxVal;
}

WAVEFORM  wave_CC_POTI01
{
	LABEL   "Position over time (actual value)|de|Position-Zeit (Istwert)"/*[TXT_wave_CC_POTI01]*/;
	Y_AXIS 	yaxis_CC_POTI;
	TYPE    YT
	{
	    X_INITIAL    var_CC_POTI01_StartX;
		X_INCREMENT  var_CC_POTI01_StepX;
		NUMBER_OF_POINTS    CC_POTI_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_POTI01
		}
	}
	EMPHASIS  IF (var_CC_POTI_Selector == CC_POTI01) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_POTI01_breaks
{
	LABEL   "Breaks in position over time (actual value)|de|Unterbrechungen in Position-Zeit (Istwert)"/*[TXT_wave_CC_POTI01_breaks]*/;
	Y_AXIS 	yaxis_CC_POTI;
	TYPE    VERTICAL
	{
		X_VALUES     
		{  
		    arr_CC_POTI01_breaks
		}
	}
	EMPHASIS  IF (var_CC_POTI_Selector == CC_POTI01) {TRUE;} ELSE {FALSE;}
	VALIDITY  IF (var_CC_POTI01_breaksindex > 0) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_POTI02
{
	LABEL   "Position over time (set value)|de|Position-Zeit (Sollwert)"/*[TXT_wave_CC_POTI02]*/;
	Y_AXIS 	yaxis_CC_POTI;
	TYPE    YT
	{
	    X_INITIAL    var_CC_POTI02_StartX;
		X_INCREMENT  var_CC_POTI02_StepX;
		NUMBER_OF_POINTS    CC_POTI_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_POTI02
		}
	}
	EMPHASIS  IF (var_CC_POTI_Selector == CC_POTI02) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_POTI02_breaks
{
	LABEL   "Breaks in position over time (set value)|de|Unterbrechungen in Position-Zeit (Sollwert)"/*[TXT_wave_CC_POTI02_breaks]*/;
	Y_AXIS 	yaxis_CC_POTI;
	TYPE    VERTICAL
	{
		X_VALUES     
		{  
		    arr_CC_POTI02_breaks
		}
	}
	EMPHASIS  IF (var_CC_POTI_Selector == CC_POTI02) {TRUE;} ELSE {FALSE;}
	VALIDITY  IF (var_CC_POTI02_breaksindex > 0) {TRUE;} ELSE {FALSE;}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MENUES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU menu_CC_POTI main page of CC POTI
// 
MENU menu_CC_POTI
{
	LABEL    "Curve position over time|de|Position-Zeit-Kennlinie";  // [TXT_menu_CC_POTI];
	STYLE    WINDOW;
	ACCESS   OFFLINE;
	ITEMS
	{
	    load_CC_POTI01,
		COLUMNBREAK,
	    load_CC_POTI02,
		COLUMNBREAK,
	    "\t",  // placeholder
		ROWBREAK,
		graph_CC_POTI,
		COLUMNBREAK,
		var_CC_POTI_Selector (NO_LABEL),
		group_CC_POTI
	}
	PRE_EDIT_ACTIONS
	{
	    calc_CC_POTI()
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU group_CC_POTI  - properties of selected CC POTI
// 
MENU group_CC_POTI
{
	LABEL    "";
	STYLE    GROUP;
	ACCESS   OFFLINE;
	ITEMS
	{
	    SELECT (var_CC_POTI_Selector)
		{
  			DEFAULT: var_CC_POTI01_WKN, var_CC_POTI01_KNR, var_CC_POTI01_time_created, var_CC_POTI01_time_stored, var_CC_POTI01_MinVal, var_CC_POTI01_MaxVal, var_CC_POTI01_FwVersion
  			CASE CC_POTI01: var_CC_POTI01_WKN, var_CC_POTI01_KNR, var_CC_POTI01_time_created, var_CC_POTI01_time_stored, var_CC_POTI01_MinVal, var_CC_POTI01_MaxVal, var_CC_POTI01_FwVersion
  			CASE CC_POTI02: var_CC_POTI02_WKN, var_CC_POTI02_KNR, var_CC_POTI02_time_created, var_CC_POTI02_time_stored, var_CC_POTI02_MinVal, var_CC_POTI02_MaxVal, var_CC_POTI02_FwVersion
		}
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_POTI  - calculate auxiliary values for graph CC POTI
// 
METHOD calc_CC_POTI
{
    LABEL    "calc_CC_POTI"; // [TXT_calc_CC_POTI];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    int extent;
	    // calculate properties of waveforms
	    calc_CC_POTI01();
	    calc_CC_POTI02();

        // MinValue of graph
        var_CC_POTI_MinVal= 65535;
		if (var_CC_POTI01_MinVal < var_CC_POTI_MinVal) var_CC_POTI_MinVal= var_CC_POTI01_MinVal;
		if (var_CC_POTI02_MinVal < var_CC_POTI_MinVal) var_CC_POTI_MinVal= var_CC_POTI02_MinVal;
        // MaxValue of graph
        var_CC_POTI_MaxVal= -65536;
		if (var_CC_POTI01_MaxVal > var_CC_POTI_MaxVal) var_CC_POTI_MaxVal= var_CC_POTI01_MaxVal;
		if (var_CC_POTI02_MaxVal > var_CC_POTI_MaxVal) var_CC_POTI_MaxVal= var_CC_POTI02_MaxVal;

		// adjust Min-/MaxVal for better display
		extent= (var_CC_POTI_MaxVal - var_CC_POTI_MinVal)/CC_RANGE_EXTENT_DIVIDER;
		var_CC_POTI_MaxVal += extent;
		var_CC_POTI_MinVal -= extent;
		
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining init_CC_POTI01  - initialize CC POTI01 data
// 
METHOD init_CC_POTI01
{
    LABEL    "init_CC_POTI01"; // [TXT_init_CC_POTI01];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		
		for (i= 0; i < CC_POTI_ARRAY_SIZE; i++)
		{
		    arr_CC_POTI01_breaks[i]= 0;
		}
		var_CC_POTI01_breaksindex= 0;
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_POTI01  - calculate auxiliary values for CC POTI01
// 
METHOD calc_CC_POTI01
{
    LABEL    "calc_CC_POTI01"; // [TXT_calc_CC_POTI01];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// adjust x-axis properties, when not initialized
		if (var_CC_POTI01_StartX == 0) var_CC_POTI01_StartX= -10;
		if (var_CC_POTI01_StepX == 0) var_CC_POTI01_StepX= CC_POTI_SKIP;

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_POTI01_MinVal= 65535;
        var_CC_POTI01_MaxVal= -65536;
		
		for (i= 0; i < CC_POTI_ARRAY_SIZE; i++)
		{
		    if (arr_CC_POTI01[i] < var_CC_POTI01_MinVal) var_CC_POTI01_MinVal= arr_CC_POTI01[i];
		    if (arr_CC_POTI01[i] > var_CC_POTI01_MaxVal) var_CC_POTI01_MaxVal= arr_CC_POTI01[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining init_CC_POTI02  - initialize CC POTI02 data
// 
METHOD init_CC_POTI02
{
    LABEL    "init_CC_POTI02"; // [TXT_init_CC_POTI02];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		
		for (i= 0; i < CC_POTI_ARRAY_SIZE; i++)
		{
		    arr_CC_POTI02_breaks[i]= 0;
		}
		var_CC_POTI02_breaksindex= 0;
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_POTI02  - calculate auxiliary values for CC POTI02
// 
METHOD calc_CC_POTI02
{
    LABEL    "calc_CC_POTI02"; // [TXT_calc_CC_POTI02];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// adjust x-axis properties, when not initialized
		if (var_CC_POTI02_StartX == 0) var_CC_POTI02_StartX= -10;
		if (var_CC_POTI02_StepX == 0) var_CC_POTI02_StepX= CC_POTI_SKIP;

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_POTI02_MinVal= 65535;
        var_CC_POTI02_MaxVal= -65536;
		
		for (i= 0; i < CC_POTI_ARRAY_SIZE; i++)
		{
		    if (arr_CC_POTI02[i] < var_CC_POTI02_MinVal) var_CC_POTI02_MinVal= arr_CC_POTI02[i];
		    if (arr_CC_POTI02[i] > var_CC_POTI02_MaxVal) var_CC_POTI02_MaxVal= arr_CC_POTI02[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_POTI01  - load CC file CCPOTI01.BIN
// 
METHOD load_CC_POTI01
{
    LABEL    "Load chart position over time (actual value)|de|Laden Position-Zeit-Kennlinie (Istwert)"; // [TXT_load_CC_POTI01];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// initialize POTI01 data
		init_CC_POTI01();
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_POTI01
			sz= "Read file " CC_POTI01_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_POTI01();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_POTI();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_POTI02  - load CC file CCPOTI02.BIN
// 
METHOD load_CC_POTI02
{
    LABEL    "Load chart position over time (set value)|de|Laden Position-Zeit-Kennlinie (Sollwert)"; // [TXT_load_CC_POTI02];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// initialize POTI02 data
		init_CC_POTI02();
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_POTI02
			sz= "Read file " CC_POTI02_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_POTI02();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_POTI();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_POTI01  -  read and parse CC file
// 
METHOD read_CC_POTI01
{
    LABEL    "Read file CCPOTI01.BIN|de|Lesen Datei CCPOTI01.BIN"; // [TXT_read_CC_POTI01];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  stepx;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		int arr_CC_tmp[CC_POTI_DATA_COUNT];
		int i;
		int mval;
		int isbreak;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCPOTI01";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_POTI01(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_POTI01_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_POTI01(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_POTI01(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_POTI01(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(POTI01, index);
					    LOAD_CC_TYPE(POTI01, index);
						LOAD_CC_NUMBER(POTI01, index);
						LOAD_CC_KNR(POTI01, index);
						LOAD_CC_WKN(POTI01, index);
						LOAD_CC_NAME(POTI01, index);
						LOAD_CC_TIME_STORED(POTI01, index);
						LOAD_CC_TIME_CREATED(POTI01, index);
						LOAD_CC_NUMVALUES(POTI01, index);
						LOAD_CC_BPV(POTI01, index);
						LOAD_CC_STARTX(POTI01, index);
						LOAD_CC_STEPX(POTI01, index);
						LOAD_CC_SETTING(POTI01, index, 1);
						LOAD_CC_SETTING(POTI01, index, 2);
						LOAD_CC_SETTING(POTI01, index, 3);
						LOAD_CC_SETTING(POTI01, index, 4);
						// load FW-Version (only from CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(POTI01, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(POTI01);
						// adjust x-axis properties
						// x-axis from -NumValues/ValuesPerHour .. 0h, step CC_POTI_SKIP/ValuesPerHour
						var_CC_POTI01_StartX= -1.0*var_CC_POTI01_NumValues*var_CC_POTI01_StepX/3600;
						var_CC_POTI01_StepX= 1.0*CC_POTI_SKIP*var_CC_POTI01_StepX/3600;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_POTI01_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(tmp, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(tmp, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(tmp, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_POTI01_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_POTI01_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_POTI01(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

        // data block is ring buffer; Setting1 gives index of next write access (--> oldest data value)
		n= 0;
		for (index= var_CC_POTI01_Setting1; index < var_CC_POTI01_NumValues; index += CC_POTI_SKIP)
		{
			// filter and mark break position and calculate mean value to find extrema value
			mval= 0; 
			isbreak= 0;
			for (i= 0; i < CC_POTI_SKIP; i++) 
			{ 
				// check for and ignore special values (store x-value as break and replace value with preceding value or 0)
				if (arr_CC_tmp[index+i] == CC_POTI_IGNOREDVAL) 
				{
				    isbreak= 1;
					if ((index+i) > 0) arr_CC_tmp[index+i]= arr_CC_tmp[index+i-1]; else arr_CC_tmp[index+i]= 0; 
				}
			    mval += arr_CC_tmp[index+i]/CC_POTI_SKIP; 
			}
			
			// if mval positive --> find maximum
			if (mval >= 0)
			{
				for (i= 0; i < CC_POTI_SKIP; i++)
				{
					// if sample is higher than mean value --> extrema
				    if (arr_CC_tmp[index+i] > mval) mval= arr_CC_tmp[index+i];
				}
			}
			// if mval negative --> find minimum
			else
			{
				for (i= 0; i < CC_POTI_SKIP; i++)
				{
					// if sample is lower than mean value --> extrema
				    if (arr_CC_tmp[index+i] < mval) mval= arr_CC_tmp[index+i];
				}
			}
			// store break position
			if (1 == isbreak)
			{
				arr_CC_POTI01_breaks[var_CC_POTI01_breaksindex++]= var_CC_POTI01_StartX + n * var_CC_POTI01_StepX;
			}
			// assign extrema value and increment array index (!)
			arr_CC_POTI01[n++]= mval;
		}		
		for (index -= var_CC_POTI01_NumValues; index < var_CC_POTI01_Setting1 && index < var_CC_POTI01_NumValues; index += CC_POTI_SKIP)
		{
			// filter and mark break position and calculate mean value to find extrema value
			mval= 0; 
			isbreak= 0;
			for (i= 0; i < CC_POTI_SKIP; i++) 
			{
				// check for and ignore special values (store x-value as break and replace value with preceding value or 0)
				if (arr_CC_tmp[index+i] == CC_POTI_IGNOREDVAL) 
				{
				    isbreak= 1;
					if ((index+i) > 0) arr_CC_tmp[index+i]= arr_CC_tmp[index+i-1]; else arr_CC_tmp[index+i]= 0; 
				}
			    mval += arr_CC_tmp[index+i]/CC_POTI_SKIP;
			}
			// if mval positive --> find maximum
			if (mval >= 0)
			{
				for (i= 0; i < CC_POTI_SKIP; i++)
				{
					// if sample is higher than mean value --> extrema
				    if (arr_CC_tmp[index+i] > mval) mval= arr_CC_tmp[index+i];
				}
			}
			// if mval negative --> find minimum
			else
			{
				for (i= 0; i < CC_POTI_SKIP; i++)
				{
					// if sample is lower than mean value --> extrema
				    if (arr_CC_tmp[index+i] < mval) mval= arr_CC_tmp[index+i];
				}
			}
			// store break position
			if (1 == isbreak)
			{
				arr_CC_POTI01_breaks[var_CC_POTI01_breaksindex++]= var_CC_POTI01_StartX + n * var_CC_POTI01_StepX;
			}
			// assign extrema value and increment array index (!)
			arr_CC_POTI01[n++]= mval;
		}

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_POTI02  -  read and parse CC file
// 
METHOD read_CC_POTI02
{
    LABEL    "Read file CCPOTI02.BIN|de|Lesen Datei CCPOTI02.BIN"; // [TXT_read_CC_POTI02];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		int arr_CC_tmp[CC_POTI_DATA_COUNT];
		int i;
		int mval;
		int isbreak;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCPOTI02";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_POTI02(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_POTI02_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_POTI02(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_POTI02(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_POTI02(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(POTI02, index);
					    LOAD_CC_TYPE(POTI02, index);
						LOAD_CC_NUMBER(POTI02, index);
						LOAD_CC_KNR(POTI02, index);
						LOAD_CC_WKN(POTI02, index);
						LOAD_CC_NAME(POTI02, index);
						LOAD_CC_TIME_STORED(POTI02, index);
						LOAD_CC_TIME_CREATED(POTI02, index);
						LOAD_CC_NUMVALUES(POTI02, index);
						LOAD_CC_BPV(POTI02, index);
						LOAD_CC_STARTX(POTI02, index);
						LOAD_CC_STEPX(POTI02, index);
						LOAD_CC_SETTING(POTI02, index, 1);
						LOAD_CC_SETTING(POTI02, index, 2);
						LOAD_CC_SETTING(POTI02, index, 3);
						LOAD_CC_SETTING(POTI02, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(POTI02, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(POTI02);
						// adjust x-axis properties
						// x-axis from -NumValues/ValuesPerHour .. 0h, step CC_POTI_SKIP/ValuesPerHour
						var_CC_POTI02_StartX= -1.0*var_CC_POTI02_NumValues*var_CC_POTI02_StepX/3600;
						var_CC_POTI02_StepX= 1.0*CC_POTI_SKIP*var_CC_POTI02_StepX/3600;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_POTI02_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(tmp, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(tmp, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(tmp, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_POTI02_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_POTI02_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_POTI02(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

        // data block is ring buffer; Setting1 gives index of next write access (--> oldest data value)
		n= 0;
		for (index= var_CC_POTI02_Setting1; index < var_CC_POTI02_NumValues; index += CC_POTI_SKIP)
		{
			// filter and mark break position and calculate mean value to find extrema value
			mval= 0; 
			isbreak= 0;
			for (i= 0; i < CC_POTI_SKIP; i++) 
			{ 
				// check for and ignore special values (store x-value as break and replace value with preceding value or 0)
				if (arr_CC_tmp[index+i] == CC_POTI_IGNOREDVAL) 
				{
				    isbreak= 1;
					if ((index+i) > 0) arr_CC_tmp[index+i]= arr_CC_tmp[index+i-1]; else arr_CC_tmp[index+i]= 0; 
				}
			    mval += arr_CC_tmp[index+i]/CC_POTI_SKIP; 
			}
			
			// if mval positive --> find maximum
			if (mval >= 0)
			{
				for (i= 0; i < CC_POTI_SKIP; i++)
				{
					// if sample is higher than mean value --> extrema
				    if (arr_CC_tmp[index+i] > mval) mval= arr_CC_tmp[index+i];
				}
			}
			// if mval negative --> find minimum
			else
			{
				for (i= 0; i < CC_POTI_SKIP; i++)
				{
					// if sample is lower than mean value --> extrema
				    if (arr_CC_tmp[index+i] < mval) mval= arr_CC_tmp[index+i];
				}
			}
			// store break position
			if (1 == isbreak)
			{
				arr_CC_POTI02_breaks[var_CC_POTI02_breaksindex++]= var_CC_POTI02_StartX + n * var_CC_POTI02_StepX;
			}
			// assign extrema value and increment array index (!)
			arr_CC_POTI02[n++]= mval;
		}		
		for (index -= var_CC_POTI02_NumValues; index < var_CC_POTI02_Setting1 && index < var_CC_POTI02_NumValues; index += CC_POTI_SKIP)
		{
			// filter and mark break position and calculate mean value to find extrema value
			mval= 0; 
			isbreak= 0;
			for (i= 0; i < CC_POTI_SKIP; i++) 
			{
				// check for and ignore special values (store x-value as break and replace value with preceding value or 0)
				if (arr_CC_tmp[index+i] == CC_POTI_IGNOREDVAL) 
				{
				    isbreak= 1;
					if ((index+i) > 0) arr_CC_tmp[index+i]= arr_CC_tmp[index+i-1]; else arr_CC_tmp[index+i]= 0; 
				}
			    mval += arr_CC_tmp[index+i]/CC_POTI_SKIP;
			}
			// if mval positive --> find maximum
			if (mval >= 0)
			{
				for (i= 0; i < CC_POTI_SKIP; i++)
				{
					// if sample is higher than mean value --> extrema
				    if (arr_CC_tmp[index+i] > mval) mval= arr_CC_tmp[index+i];
				}
			}
			// if mval negative --> find minimum
			else
			{
				for (i= 0; i < CC_POTI_SKIP; i++)
				{
					// if sample is lower than mean value --> extrema
				    if (arr_CC_tmp[index+i] < mval) mval= arr_CC_tmp[index+i];
				}
			}
			// store break position
			if (1 == isbreak)
			{
				arr_CC_POTI02_breaks[var_CC_POTI02_breaksindex++]= var_CC_POTI02_StartX + n * var_CC_POTI02_StepX;
			}
			// assign extrema value and increment array index (!)
			arr_CC_POTI02[n++]= mval;
		}

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
