//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2011). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for characteristic curve TETI tunneled over ServiceChannel 
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define CC_TETI     1
#define CC_TETI_DATA_COUNT	17520
#define CC_TETI_SKIP        12  // one per 6h 
#define CC_TETI_ARRAY_SIZE	(CC_TETI_DATA_COUNT/CC_TETI_SKIP)
#define CC_TETI_XUNIT        "d"
#define CC_TETI_YUNIT        SELECT (var_CC_TETI_UnitSelector ) {CASE CC_UNIT_C: [TXT_TYP_678];CASE CC_UNIT_F: [TXT_TYP_679];}
#define CC_TETI_IGNOREDVAL   0x80

#define CC_TETI01   1
#define CC_TETI02   2
//#define CC_TETI03   3
//#define CC_TETI04   4

#define CC_UNIT_C   1
#define CC_UNIT_F   2

#define CC_TETI01_FILE   "CCTETI01.BIN"
#define CC_TETI02_FILE   "CCTETI02.BIN"
//#define CC_TETI03_FILE   "CCTETI03.BIN"
//#define CC_TETI04_FILE   "CCTETI04.BIN"

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VARIABLE var_CC_TETI_Selector
{
    LABEL    "Curve Selection|de|Kennlinienauswahl";  // [TXT_var_CC_Selector];
	CLASS    LOCAL;
	TYPE     ENUMERATED (2)
	{
        DEFAULT_VALUE	1;
        { CC_TETI01, "Temperature over time (controller)|de|Temperatur-Zeit (Steuerung)"/*[TXT_wave_CC_TETI01]*/ }
        IF (PRM_606 == 1) {{ CC_TETI02, "Temperature over time (MWG)|de|Temperatur-Zeit (MWG)"/*[TXT_wave_CC_TETI02]*/ }}
        // ,{ CC_TETI03, "Temperature over time (motor)|de|Temperatur-Zeit (Motor)"/*[TXT_wave_CC_TETI03]*/ }
        // ,{ CC_TETI04, "Temperature over time (gearbox)|de|Temperatur-Zeit (Getriebe)"/*[TXT_wave_CC_TETI04]*/ }
	}
}

VARIABLE var_CC_TETI_UnitSelector
{
    LABEL    "Unit|de|Einheit";  // [TXT_var_CC_UnitSelector];
	CLASS    LOCAL;
	TYPE     ENUMERATED (2)
	{
        DEFAULT_VALUE	1;
        { CC_UNIT_C, [TXT_TYP_678] /* "C" */}
        ,{ CC_UNIT_F, [TXT_TYP_679] /* "F" */ }
	}
	POST_EDIT_ACTIONS
	{
	    calc_CC_TETI()
	}
}

var_CC_TETI_Value LIKE VARIABLE var_CC_data_int8{REDEFINE CONSTANT_UNIT CC_TETI_YUNIT;}
var_CC_TETI_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TETI_YUNIT;}
var_CC_TETI_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TETI_YUNIT;}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC TETI01
// 
var_CC_TETI01_Version LIKE VARIABLE var_CC_Version{}
var_CC_TETI01_Type LIKE VARIABLE var_CC_Type{}
var_CC_TETI01_Number LIKE VARIABLE var_CC_Number{}
var_CC_TETI01_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_TETI01_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_TETI01_Name LIKE VARIABLE var_CC_Name{}
var_CC_TETI01_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_TETI01_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_TETI01_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_TETI01_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_TETI01_StartX LIKE VARIABLE var_CC_StartX{}
var_CC_TETI01_StepX LIKE VARIABLE var_CC_StepX{}
var_CC_TETI01_Setting1 LIKE VARIABLE var_CC_Setting1{}
var_CC_TETI01_Setting2 LIKE VARIABLE var_CC_Setting2{}
var_CC_TETI01_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_TETI01_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_TETI01_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_TETI01_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_TETI01_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TETI_YUNIT;}
var_CC_TETI01_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TETI_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TETI01 values
// 
ARRAY arr_CC_TETI01
{
    LABEL    "CC TETI01 values";  // [TXT_arr_CC_TETI01];
	NUMBER_OF_ELEMENTS    CC_TETI_ARRAY_SIZE;
	TYPE    var_CC_TETI_Value;
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TETI01 original values
// 
ARRAY arr_CC_TETI01_orig
{
    LABEL    "CC TETI01 original values";  // [TXT_arr_CC_TETI01_orig];
	NUMBER_OF_ELEMENTS    CC_TETI_ARRAY_SIZE;
	TYPE    var_CC_TETI_Value;
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TETI01 breaks
// 
ARRAY arr_CC_TETI01_breaks
{
    LABEL    "CC TETI01 breaks";  // [TXT_arr_CC_TETI01_breaks];
	NUMBER_OF_ELEMENTS    CC_TETI_ARRAY_SIZE;
	TYPE    var_CC_StepX;
}
VARIABLE var_CC_TETI01_breaksindex
{
	LABEL		"CC TETI01 breaks index";  // [TXT_var_CC_TETI01_breaksindex];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(2)
	{
	    DEFAULT_VALUE   0;
	}
}


//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC TETI02
// 
var_CC_TETI02_Version LIKE VARIABLE var_CC_Version{}
var_CC_TETI02_Type LIKE VARIABLE var_CC_Type{}
var_CC_TETI02_Number LIKE VARIABLE var_CC_Number{}
var_CC_TETI02_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_TETI02_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_TETI02_Name LIKE VARIABLE var_CC_Name{}
var_CC_TETI02_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_TETI02_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_TETI02_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_TETI02_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_TETI02_StartX LIKE VARIABLE var_CC_StartX{}
var_CC_TETI02_StepX LIKE VARIABLE var_CC_StepX{}
var_CC_TETI02_Setting1 LIKE VARIABLE var_CC_Setting1{}
var_CC_TETI02_Setting2 LIKE VARIABLE var_CC_Setting2{}
var_CC_TETI02_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_TETI02_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_TETI02_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_TETI02_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_TETI02_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TETI_YUNIT;}
var_CC_TETI02_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TETI_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TETI02 values
// 
ARRAY arr_CC_TETI02
{
    LABEL    "CC TETI02 values";  // [TXT_arr_CC_TETI02];
	NUMBER_OF_ELEMENTS    CC_TETI_ARRAY_SIZE;
	TYPE    var_CC_TETI_Value;
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TETI02 original values
// 
ARRAY arr_CC_TETI02_orig
{
    LABEL    "CC TETI02 original values";  // [TXT_arr_CC_TETI02_orig];
	NUMBER_OF_ELEMENTS    CC_TETI_ARRAY_SIZE;
	TYPE    var_CC_TETI_Value;
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TETI02 breaks
// 
ARRAY arr_CC_TETI02_breaks
{
    LABEL    "CC TETI02 breaks";  // [TXT_arr_CC_TETI02_breaks];
	NUMBER_OF_ELEMENTS    CC_TETI_ARRAY_SIZE;
	TYPE    var_CC_StepX;
}
VARIABLE var_CC_TETI02_breaksindex
{
	LABEL		"CC TETI02 breaks valid";  // [TXT_var_CC_TETI02_breaksindex];
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER(2)
	{
	    DEFAULT_VALUE   0;
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GRAPHS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

GRAPH graph_CC_TETI
{
	LABEL 	"Characteristic curves temperature over time|de|Temperatur-Zeit-Kennlinien"; // [TXT_graph_CC_TETI];
	X_AXIS 	xaxis_CC_TETI;
	WIDTH	X_SMALL;
	MEMBERS
	{
		WAVEFORM1, wave_CC_TETI01;
		IF (var_CC_TETI01_breaksindex > 0) { WAVEFORM2, wave_CC_TETI01_breaks; }
		IF (PRM_606 == 1) { WAVEFORM3, wave_CC_TETI02; }
		IF ((PRM_606 == 1) && (var_CC_TETI02_breaksindex > 0)) { WAVEFORM4, wave_CC_TETI02_breaks; }
	}
}

AXIS  	xaxis_CC_TETI
{
    LABEL  "Days before|de|Tage zurck"; // [TXT_xaxis_CC_TETI];
	CONSTANT_UNIT    CC_TETI_XUNIT;
	MIN_VALUE    var_CC_TETI01_StartX;
	MAX_VALUE    0;
}

AXIS  	yaxis_CC_TETI
{
    LABEL  "Temperature|de|Temperatur";
	// no CONSTANT_UNIT -- UNIT given by MIN_/MAX_VALUE
	MIN_VALUE    var_CC_TETI_MinVal;
	MAX_VALUE    var_CC_TETI_MaxVal;
}

WAVEFORM  wave_CC_TETI01
{
	LABEL   "Temperature over time (controller)|de|Temperatur-Zeit (Steuerung)"/*[TXT_wave_CC_TETI01]*/;
	Y_AXIS 	yaxis_CC_TETI;
	TYPE    YT
	{
	    X_INITIAL    var_CC_TETI01_StartX;
		X_INCREMENT  var_CC_TETI01_StepX;
		NUMBER_OF_POINTS    CC_TETI_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_TETI01
		}
	}
	EMPHASIS  IF (var_CC_TETI_Selector == CC_TETI01) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_TETI01_breaks
{
	LABEL   "Breaks temperature over time (controller)|de|Unterbrechungen in Temperatur-Zeit (Steuerung)"/*[TXT_wave_CC_TETI01_breaks]*/;
	Y_AXIS 	yaxis_CC_TETI;
	TYPE    VERTICAL
	{
		X_VALUES     
		{  
		    arr_CC_TETI01_breaks
		}
	}
	EMPHASIS  IF (var_CC_TETI_Selector == CC_TETI01) {TRUE;} ELSE {FALSE;}
	VALIDITY  IF (var_CC_TETI01_breaksindex > 0) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_TETI02
{
	LABEL   "Temperature over time (MWG)|de|Temperatur-Zeit (MWG)"/*[TXT_wave_CC_TETI02]*/;
	Y_AXIS 	yaxis_CC_TETI;
	TYPE    YT
	{
	    X_INITIAL    var_CC_TETI02_StartX;
		X_INCREMENT  var_CC_TETI02_StepX;
		NUMBER_OF_POINTS    CC_TETI_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_TETI02
		}
	}
	EMPHASIS  IF (var_CC_TETI_Selector == CC_TETI02) {TRUE;} ELSE {FALSE;}
	VALIDITY  IF (PRM_606 == 1) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_TETI02_breaks
{
	LABEL   "Breaks temperature over time (MWG)|de|Unterbrechungen in Temperatur-Zeit (MWG)"/*[TXT_wave_CC_TETI02_breaks]*/;
	Y_AXIS 	yaxis_CC_TETI;
	TYPE    VERTICAL
	{
		X_VALUES     
		{  
		    arr_CC_TETI02_breaks
		}
	}
	EMPHASIS  IF (var_CC_TETI_Selector == CC_TETI02) {TRUE;} ELSE {FALSE;}
	VALIDITY  IF ((PRM_606 == 1) && (var_CC_TETI02_breaksindex > 0)) {TRUE;} ELSE {FALSE;}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MENUES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU menu_CC_TETI main page of CC TETI
// 
MENU menu_CC_TETI
{
	LABEL    "Curve temperature over time|de|Temperatur-Zeit-Kennlinie";  // [TXT_menu_CC_TETI];
	STYLE    WINDOW;
	ACCESS   OFFLINE;
	ITEMS
	{
	    load_CC_TETI01,
		COLUMNBREAK,
	    load_CC_TETI02,
		COLUMNBREAK,
	    "\t",  // placeholder
		ROWBREAK,
		graph_CC_TETI,
		COLUMNBREAK,
		var_CC_TETI_Selector (NO_LABEL),
		var_CC_TETI_UnitSelector,
		group_CC_TETI
	}
	PRE_EDIT_ACTIONS
	{
	    calc_CC_TETI()
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU group_CC_TETI  - properties of selected CC TETI
// 
MENU group_CC_TETI
{
	LABEL    "";
	STYLE    GROUP;
	ACCESS   OFFLINE;
	ITEMS
	{
	    SELECT (var_CC_TETI_Selector)
		{
  			DEFAULT: var_CC_TETI01_WKN, var_CC_TETI01_KNR, var_CC_TETI01_time_created, var_CC_TETI01_time_stored, var_CC_TETI01_MinVal, var_CC_TETI01_MaxVal, var_CC_TETI01_FwVersion
  			CASE CC_TETI01: var_CC_TETI01_WKN, var_CC_TETI01_KNR, var_CC_TETI01_time_created, var_CC_TETI01_time_stored, var_CC_TETI01_MinVal, var_CC_TETI01_MaxVal, var_CC_TETI01_FwVersion
  			CASE CC_TETI02: var_CC_TETI02_WKN, var_CC_TETI02_KNR, var_CC_TETI02_time_created, var_CC_TETI02_time_stored, var_CC_TETI02_MinVal, var_CC_TETI02_MaxVal, var_CC_TETI02_FwVersion
		}
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining convert_CC_TETI_value()
// 
METHOD convert_CC_TETI_value (int value)
{
    LABEL    "convert_CC_TETI_value"; // [TXT_convert_CC_TETI_value];
	ACCESS    OFFLINE;
	TYPE      int;
    DEFINITION
    {
		switch (var_CC_TETI_UnitSelector)
		{
			default:
			case CC_UNIT_C:
				return (value);
				break;
			case CC_UNIT_F:
				return ((value*9)/5) + 32;
				break;
		}
	}
}

		//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TETI  - calculate auxiliary values for graph CC TETI
// 
METHOD calc_CC_TETI
{
    LABEL    "calc_CC_TETI"; // [TXT_calc_CC_TETI];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    int extent;
	    // calculate properties of waveforms
	    calc_CC_TETI01();
	    if (PRM_606 == 1) calc_CC_TETI02();

        // MinValue of graph
        var_CC_TETI_MinVal= 65535;
		if (var_CC_TETI01_MinVal < var_CC_TETI_MinVal) var_CC_TETI_MinVal= var_CC_TETI01_MinVal;
		if (PRM_606 == 1) { if (var_CC_TETI02_MinVal < var_CC_TETI_MinVal) var_CC_TETI_MinVal= var_CC_TETI02_MinVal; }

        // MaxValue of graph
        var_CC_TETI_MaxVal= -65536;
		if (var_CC_TETI01_MaxVal > var_CC_TETI_MaxVal) var_CC_TETI_MaxVal= var_CC_TETI01_MaxVal;
		if (PRM_606 == 1) { if (var_CC_TETI02_MaxVal > var_CC_TETI_MaxVal) var_CC_TETI_MaxVal= var_CC_TETI02_MaxVal; }

		// adjust Min-/MaxVal for better display
		extent= (var_CC_TETI_MaxVal - var_CC_TETI_MinVal)/CC_RANGE_EXTENT_DIVIDER;
		var_CC_TETI_MaxVal += extent;
		var_CC_TETI_MinVal -= extent;
		
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining init_CC_TETI01  - initialize CC TETI01 data
// 
METHOD init_CC_TETI01
{
    LABEL    "init_CC_TETI01"; // [TXT_init_CC_TETI01];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		
		for (i= 0; i < CC_TETI_ARRAY_SIZE; i++)
		{
		    arr_CC_TETI01_breaks[i]= 0;
		}
		var_CC_TETI01_breaksindex= 0;
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TETI01  - calculate auxiliary values for CC TETI01
// 
METHOD calc_CC_TETI01
{
    LABEL    "calc_CC_TETI01"; // [TXT_calc_CC_TETI01];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// adjust x-axis properties, when not initialized
		if (var_CC_TETI01_StartX == 0) var_CC_TETI01_StartX= -365;
		if (var_CC_TETI01_StepX == 0) var_CC_TETI01_StepX= 1.0*CC_TETI_SKIP/(2*24);

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_TETI01_MinVal= 65535;
        var_CC_TETI01_MaxVal= -65536;
		for (i= 0; i < CC_TETI_ARRAY_SIZE; i++)
		{
			// re-calculate displayed values
			arr_CC_TETI01[i]= convert_CC_TETI_value(arr_CC_TETI01_orig[i]);
			// find minimum
		    if (arr_CC_TETI01[i] < var_CC_TETI01_MinVal) var_CC_TETI01_MinVal= arr_CC_TETI01[i];
			// find maximum
		    if (arr_CC_TETI01[i] > var_CC_TETI01_MaxVal) var_CC_TETI01_MaxVal= arr_CC_TETI01[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining init_CC_TETI02  - initialize CC TETI02 data
// 
METHOD init_CC_TETI02
{
    LABEL    "init_CC_TETI02"; // [TXT_init_CC_TETI02];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		
		for (i= 0; i < CC_TETI_ARRAY_SIZE; i++)
		{
		    arr_CC_TETI02_breaks[i]= 0;
		}
		var_CC_TETI02_breaksindex= 0;
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TETI02  - calculate auxiliary values for CC TETI02
// 
METHOD calc_CC_TETI02
{
    LABEL    "calc_CC_TETI02"; // [TXT_calc_CC_TETI02];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// adjust x-axis properties, when not initialized
		if (var_CC_TETI02_StartX == 0) var_CC_TETI02_StartX= -365;
		if (var_CC_TETI02_StepX == 0) var_CC_TETI02_StepX= 1.0*CC_TETI_SKIP/(2*24);

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_TETI02_MinVal= 65535;
        var_CC_TETI02_MaxVal= -65536;
		for (i= 0; i < CC_TETI_ARRAY_SIZE; i++)
		{
			// re-calculate displayed values
			arr_CC_TETI02[i]= convert_CC_TETI_value(arr_CC_TETI02_orig[i]);
			// find minimum
		    if (arr_CC_TETI02[i] < var_CC_TETI02_MinVal) var_CC_TETI02_MinVal= arr_CC_TETI02[i];
			// find maximum
		    if (arr_CC_TETI02[i] > var_CC_TETI02_MaxVal) var_CC_TETI02_MaxVal= arr_CC_TETI02[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_TETI01  - load CC file CCTETI01.BIN
// 
METHOD load_CC_TETI01
{
    LABEL    "Load chart temperature over time (controller)|de|Laden Temperatur-Zeit-Kennlinie (Steuerung)"; // [TXT_load_CC_TETI01];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// initialize TETI01 data
		init_CC_TETI01();
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_TETI01
			sz= "Read file " CC_TETI01_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_TETI01();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_TETI();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_TETI02  - load CC file CCTETI02.BIN
// 
METHOD load_CC_TETI02
{
    LABEL    "Load chart temperature over time (MWG)|de|Laden Temperatur-Zeit-Kennlinie (MWG)"; // [TXT_load_CC_TETI02];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// initialize TETI02 data
		init_CC_TETI02();
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_TETI02
			sz= "Read file " CC_TETI02_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_TETI02();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_TETI();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_TETI01  -  read and parse CC file
// 
METHOD read_CC_TETI01
{
    LABEL    "Read file CCTETI01.BIN|de|Lesen Datei CCTETI01.BIN"; // [TXT_read_CC_TETI01];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		int arr_CC_tmp[CC_TETI_DATA_COUNT];
		int i;
		int mval;
		int isbreak;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCTETI01";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_TETI01(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_TETI01_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_TETI01(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_TETI01(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_TETI01(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(TETI01, index);
					    LOAD_CC_TYPE(TETI01, index);
						LOAD_CC_NUMBER(TETI01, index);
						LOAD_CC_KNR(TETI01, index);
						LOAD_CC_WKN(TETI01, index);
						LOAD_CC_NAME(TETI01, index);
						LOAD_CC_TIME_STORED(TETI01, index);
						LOAD_CC_TIME_CREATED(TETI01, index);
						LOAD_CC_NUMVALUES(TETI01, index);
						LOAD_CC_BPV(TETI01, index);
						LOAD_CC_STARTX(TETI01, index);
						LOAD_CC_STEPX(TETI01, index);
						LOAD_CC_SETTING(TETI01, index, 1);
						LOAD_CC_SETTING(TETI01, index, 2);
						LOAD_CC_SETTING(TETI01, index, 3);
						LOAD_CC_SETTING(TETI01, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(TETI01, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(TETI01);
						// adjust x-axis properties
						// x-axis from -365d (-NumValues/ValuesPerHour) .. 0d, step 1/4d (CC_TETI_SKIP/ValuesPerHour)
						var_CC_TETI01_StartX= -1.0*var_CC_TETI01_NumValues/(2*24);
						var_CC_TETI01_StepX= 1.0*CC_TETI_SKIP/(2*24);

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_TETI01_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(tmp, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(tmp, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(tmp, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_TETI01_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_TETI01_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_TETI01(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

        // data block is ring buffer; Setting1 gives index of next write access (--> oldest data value)
		n= 0;
		for (index= var_CC_TETI01_Setting1; index < var_CC_TETI01_NumValues; index += CC_TETI_SKIP)
		{
			// filter and mark break position and calculate mean value to find extrema value
			mval= 0; 
			isbreak= 0;
			for (i= 0; i < CC_TETI_SKIP; i++) 
			{ 
				// check for and ignore special values (store x-value as break and replace value with preceding value or 0)
				if (arr_CC_tmp[index+i] == CC_TETI_IGNOREDVAL) 
				{
				    isbreak= 1;
					if ((index+i) > 0) arr_CC_tmp[index+i]= arr_CC_tmp[index+i-1]; else arr_CC_tmp[index+i]= 0; 
				}
			    mval += arr_CC_tmp[index+i]/CC_TETI_SKIP; 
			}
			
			// if mval positive --> find maximum
			if (mval >= 0)
			{
				for (i= 0; i < CC_TETI_SKIP; i++)
				{
					// if sample is higher than mean value --> extrema
				    if (arr_CC_tmp[index+i] > mval) mval= arr_CC_tmp[index+i];
				}
			}
			// if mval negative --> find minimum
			else
			{
				for (i= 0; i < CC_TETI_SKIP; i++)
				{
					// if sample is lower than mean value --> extrema
				    if (arr_CC_tmp[index+i] < mval) mval= arr_CC_tmp[index+i];
				}
			}
			// store break position
			if (1 == isbreak)
			{
				arr_CC_TETI01_breaks[var_CC_TETI01_breaksindex++]= var_CC_TETI01_StartX + n * var_CC_TETI01_StepX;
			}
			// assign extrema value and increment array index (!)
			arr_CC_TETI01_orig[n++]= mval;
		}		
		for (index -= var_CC_TETI01_NumValues; index < var_CC_TETI01_Setting1 && index < var_CC_TETI01_NumValues; index += CC_TETI_SKIP)
		{
			// filter and mark break position and calculate mean value to find extrema value
			mval= 0; 
			isbreak= 0;
			for (i= 0; i < CC_TETI_SKIP; i++) 
			{
				// check for and ignore special values (store x-value as break and replace value with preceding value or 0)
				if (arr_CC_tmp[index+i] == CC_TETI_IGNOREDVAL) 
				{
				    isbreak= 1;
					if ((index+i) > 0) arr_CC_tmp[index+i]= arr_CC_tmp[index+i-1]; else arr_CC_tmp[index+i]= 0; 
				}
			    mval += arr_CC_tmp[index+i]/CC_TETI_SKIP;
			}
			// if mval positive --> find maximum
			if (mval >= 0)
			{
				for (i= 0; i < CC_TETI_SKIP; i++)
				{
					// if sample is higher than mean value --> extrema
				    if (arr_CC_tmp[index+i] > mval) mval= arr_CC_tmp[index+i];
				}
			}
			// if mval negative --> find minimum
			else
			{
				for (i= 0; i < CC_TETI_SKIP; i++)
				{
					// if sample is lower than mean value --> extrema
				    if (arr_CC_tmp[index+i] < mval) mval= arr_CC_tmp[index+i];
				}
			}
			// store break position
			if (1 == isbreak)
			{
				arr_CC_TETI01_breaks[var_CC_TETI01_breaksindex++]= var_CC_TETI01_StartX + n * var_CC_TETI01_StepX;
			}
			// assign extrema value and increment array index (!)
			arr_CC_TETI01_orig[n++]= mval;
		}

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_TETI02  -  read and parse CC file
// 
METHOD read_CC_TETI02
{
    LABEL    "Read file CCTETI02.BIN|de|Lesen Datei CCTETI02.BIN"; // [TXT_read_CC_TETI02];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		int arr_CC_tmp[CC_TETI_DATA_COUNT];
		int i;
		int mval;
		int isbreak;
		
		// read presence of MWG
		ReadCommand(read_PRM_606);
		if (PRM_606 != 1)
		{
		    // no MWG available --> cancel
			PUT_MESSAGE ("MWG not present -- reading curve cancelled!|de|Kein MWG verfgbar -- Abbruch!"); // [TXT_CC_TETI_NoMWG]
			return (SC_NOT_FOUND);
		}
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCTETI02";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_TETI02(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_TETI02_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_TETI02(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_TETI02(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_TETI02(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(TETI02, index);
					    LOAD_CC_TYPE(TETI02, index);
						LOAD_CC_NUMBER(TETI02, index);
						LOAD_CC_KNR(TETI02, index);
						LOAD_CC_WKN(TETI02, index);
						LOAD_CC_NAME(TETI02, index);
						LOAD_CC_TIME_STORED(TETI02, index);
						LOAD_CC_TIME_CREATED(TETI02, index);
						LOAD_CC_NUMVALUES(TETI02, index);
						LOAD_CC_BPV(TETI02, index);
						LOAD_CC_STARTX(TETI02, index);
						LOAD_CC_STEPX(TETI02, index);
						LOAD_CC_SETTING(TETI02, index, 1);
						LOAD_CC_SETTING(TETI02, index, 2);
						LOAD_CC_SETTING(TETI02, index, 3);
						LOAD_CC_SETTING(TETI02, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(TETI02, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(TETI02);
						// adjust x-axis properties
						// x-axis from -365d (-NumValues/ValuesPerHour) .. 0d, step 1/4d (CC_TETI_SKIP/ValuesPerHour)
						var_CC_TETI02_StartX= -1.0*var_CC_TETI02_NumValues/(2*24);
						var_CC_TETI02_StepX= 1.0*CC_TETI_SKIP/(2*24);

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_TETI02_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(tmp, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(tmp, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(tmp, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_TETI02_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_TETI02_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_TETI02(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

        // data block is ring buffer; Setting1 gives index of next write access (--> oldest data value)
		n= 0;
		for (index= var_CC_TETI02_Setting1; index < var_CC_TETI02_NumValues; index += CC_TETI_SKIP)
		{
			// filter and mark break position and calculate mean value to find extrema value
			mval= 0; 
			isbreak= 0;
			for (i= 0; i < CC_TETI_SKIP; i++) 
			{ 
				// check for and ignore special values (store x-value as break and replace value with preceding value or 0)
				if (arr_CC_tmp[index+i] == CC_TETI_IGNOREDVAL) 
				{
				    isbreak= 1;
					if ((index+i) > 0) arr_CC_tmp[index+i]= arr_CC_tmp[index+i-1]; else arr_CC_tmp[index+i]= 0; 
				}
			    mval += arr_CC_tmp[index+i]/CC_TETI_SKIP; 
			}
			
			// if mval positive --> find maximum
			if (mval >= 0)
			{
				for (i= 0; i < CC_TETI_SKIP; i++)
				{
					// if sample is higher than mean value --> extrema
				    if (arr_CC_tmp[index+i] > mval) mval= arr_CC_tmp[index+i];
				}
			}
			// if mval negative --> find minimum
			else
			{
				for (i= 0; i < CC_TETI_SKIP; i++)
				{
					// if sample is lower than mean value --> extrema
				    if (arr_CC_tmp[index+i] < mval) mval= arr_CC_tmp[index+i];
				}
			}
			// store break position
			if (1 == isbreak)
			{
				arr_CC_TETI02_breaks[var_CC_TETI02_breaksindex++]= var_CC_TETI02_StartX + n * var_CC_TETI02_StepX;
			}
			// assign extrema value and increment array index (!)
			arr_CC_TETI02_orig[n++]= mval;
		}		
		for (index -= var_CC_TETI02_NumValues; index < var_CC_TETI02_Setting1 && index < var_CC_TETI02_NumValues; index += CC_TETI_SKIP)
		{
			// filter and mark break position and calculate mean value to find extrema value
			mval= 0; 
			isbreak= 0;
			for (i= 0; i < CC_TETI_SKIP; i++) 
			{
				// check for and ignore special values (store x-value as break and replace value with preceding value or 0)
				if (arr_CC_tmp[index+i] == CC_TETI_IGNOREDVAL) 
				{
				    isbreak= 1;
					if ((index+i) > 0) arr_CC_tmp[index+i]= arr_CC_tmp[index+i-1]; else arr_CC_tmp[index+i]= 0; 
				}
			    mval += arr_CC_tmp[index+i]/CC_TETI_SKIP;
			}
			// if mval positive --> find maximum
			if (mval >= 0)
			{
				for (i= 0; i < CC_TETI_SKIP; i++)
				{
					// if sample is higher than mean value --> extrema
				    if (arr_CC_tmp[index+i] > mval) mval= arr_CC_tmp[index+i];
				}
			}
			// if mval negative --> find minimum
			else
			{
				for (i= 0; i < CC_TETI_SKIP; i++)
				{
					// if sample is lower than mean value --> extrema
				    if (arr_CC_tmp[index+i] < mval) mval= arr_CC_tmp[index+i];
				}
			}
			// store break position
			if (1 == isbreak)
			{
				arr_CC_TETI02_breaks[var_CC_TETI02_breaksindex++]= var_CC_TETI02_StartX + n * var_CC_TETI02_StepX;
			}
			// assign extrema value and increment array index (!)
			arr_CC_TETI02_orig[n++]= mval;
		}

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
