/* ---------------------------------------------------------------------------------------------------------------------
 * Copyright (c) AUMA Riester GmbH & Co. KG (2012). All Rights Reserved
 * ---------------------------------------------------------------------------------------------------------------------
 * Target System: SIMATIC PDM
 * ---------------------------------------------------------------------------------------------------------------------
 * (c) Sipos Aktorik GmbH, 2013 (Mathias Monse <mathias.monse@sipos.de>)
 * ---------------------------------------------------------------------------------------------------------------------
 */

/* ---------------------------------------------------------------------------------------------------------------------
 * Defines
 */
#define RC_CMD_CODE_STOP     0x00   
#define RC_CMD_CODE_OPEN     0x01        /* BIT0 */
#define RC_CMD_CODE_CLOSE    0x02        /* BIT1 */
#define RC_CMD_CODE_SETPOS   0x04        /* BIT2 */
#define RC_CMD_CODE_RESET    0x08        /* BIT3 */

#ifndef VARIABLE_STATUS_LOADED
#define VARIABLE_STATUS_LOADED        0x0010
#endif
/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  var_RC_active
 */
VARIABLE var_RC_active
{
	LABEL	"Remote control active|de|Fernbedienung aktiv"; //[TXT_var_RC_active];
	CLASS	LOCAL;
	TYPE	UNSIGNED_INTEGER(1)
	{
		DEFAULT_VALUE	0;
	}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  var_RC_onoff
 */
var_RC_onoff LIKE VARIABLE PZD_737 { 
	REDEFINE CLASS DYNAMIC; 
	REDEFINE POST_READ_ACTIONS { method_RC_status_refresh }
	}
COMMAND read_var_RC_onoff
{
	SLOT	7;
	INDEX	165;
	OPERATION	READ;
	TRANSACTION
	{
		REPLY
		{
			var_RC_onoff
		}
		REQUEST
		{
		}
	}
}
/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  var_RC_switch
 */
var_RC_switch LIKE VARIABLE	PZD_112 { REDEFINE CLASS DYNAMIC; }
COMMAND read_var_RC_switch
{
	SLOT	1;
	INDEX	140;
	OPERATION	READ;
	TRANSACTION
	{
		REPLY
		{
			var_RC_switch
		}
		REQUEST
		{
		}
	}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  var_RC_on
 */
VARIABLE var_RC_on
{
    LABEL	"Remote control set active|de|Fernbedienung aktiv setzen"; //[TXT_var_RC_on];
    CLASS	LOCAL;
    TYPE	UNSIGNED_INTEGER (1)
	{
		DEFAULT_VALUE	0;
	}
	HANDLING	READ;
}

var_RC_on_device LIKE VARIABLE var_RC_on
{
	REDEFINE CLASS	CONTAINED;
	REDEFINE HANDLING	WRITE;
}

COMMAND write_RC_on
{
	SLOT	253;
	INDEX	1;
	OPERATION	WRITE;
	TRANSACTION
	{
		REPLY
		{
		}
		REQUEST
		{
			var_RC_on_device
		}
	}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  var_RC_cmd
 */
VARIABLE var_RC_cmd
{
    LABEL	"Remote control command|de|Fernbedienung Kommando"; //[TXT_var_RC_cmd];
    CLASS	CONTAINED;
    TYPE	UNSIGNED_INTEGER (1);
	HANDLING	WRITE;
}

COMMAND write_RC_cmd
{
	SLOT	253;
	INDEX	2;
	OPERATION	WRITE;
	TRANSACTION
	{
		REPLY
		{
		}
		REQUEST
		{
			var_RC_cmd
		}
	}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  var_RC_torque_unit
 */
VARIABLE var_RC_torque_unit
{
    LABEL	"Torque unit|de|Einheit Drehmoment"; //[TXT_TST_315];
    CLASS	LOCAL;
    TYPE	ENUMERATED (2)
    {
        DEFAULT_VALUE	1;
        { 1, [TXT_TYP_705] },
        { 2, [TXT_TYP_703] },
        { 3, [TXT_TYP_704] }  
    }
    HANDLING		READ & WRITE;
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  var_RC_torque
 */
VARIABLE	var_RC_torque 
{
    LABEL		[TXT_PZD_545];
	CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER (4)
    {
        DEFAULT_VALUE	0;
    }
	CONSTANT_UNIT 	SELECT (var_RC_torque_unit) 
					{ 
						DEFAULT: PZD_380.VARIABLE_UNIT;
						CASE 1: PZD_380.VARIABLE_UNIT;
						CASE 2: PZD_146.VARIABLE_UNIT;
						CASE 3: PZD_378.VARIABLE_UNIT;
					}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  var_RC_position 	(Stellungsistwert Stellantrieb E2)
 */
var_RC_position LIKE VARIABLE	PZD_2 { 
	REDEFINE CLASS		DYNAMIC; 
	REDEFINE POST_READ_ACTIONS { method_RC_torque_refresh }
}
COMMAND read_var_RC_position
{
    SLOT		0;
    INDEX		130;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            var_RC_position    /* 2 Octets */
        }
        REQUEST
        {
        }
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  PZD_369
 */
VARIABLE PZD_369	/* Endlage AUF + Blinker */
{
    LABEL		"Endlage AUF + Blinker"; /* [TXT_PZD_369]; */
    HELP		"Endlage AUF + Blinker"; /* [HLP_PZD_369]; */
    CLASS		DIAGNOSTIC & DYNAMIC;
    TYPE		ENUMERATED (4)
    {
        DEFAULT_VALUE	0;
        { 0, "false"},
        { 1, "true"}
    }
    HANDLING		READ;
}

COMMAND read_PZD_369
{
    SLOT		3;
    INDEX		197;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_369    /* 4 Octets */
        }
        REQUEST
        {
        }
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining VARIABLE  PZD_370
 */
VARIABLE PZD_370	/* Endlage ZU + Blinker */
{
    LABEL		"Endlage ZU + Blinker"; /* [TXT_PZD_370]; */
    HELP		"Endlage ZU + Blinker"; /* [HLP_PZD_370]; */
    CLASS		DIAGNOSTIC & DYNAMIC;
    TYPE		ENUMERATED (4)
    {
        DEFAULT_VALUE	0;
        { 0, "false"},
        { 1, "true"}
    }
    HANDLING		READ;
}

COMMAND read_PZD_370
{
    SLOT		3;
    INDEX		198;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_370    /* 4 Octets */
        }
        REQUEST
        {
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  PZD_380
// 
VARIABLE PZD_380	// Drehmomentwert Richtung AUF in % des Antrieb-Nenndrehmoments
{
    LABEL		"Drehmomentwert Richtung AUF in % des Antrieb-Nenndrehmoments"; // [TXT_PZD_380];
    HELP		"Drehmomentwert Richtung AUF in % des Antrieb-Nenndrehmoments"; // [HLP_PZD_380];
    CLASS		DYNAMIC;
    TYPE		UNSIGNED_INTEGER (2)
    {
        MIN_VALUE	0;
        MAX_VALUE	127;
        DEFAULT_VALUE	0;
    }
    CONSTANT_UNIT		"%";
    HANDLING		READ;
}

COMMAND read_PZD_380
{
    SLOT		3;
    INDEX		208;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_380    // 2 Octets
        }
        REQUEST
        {
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  PZD_381
// 
VARIABLE PZD_381	// Drehmomentwert Richtung ZU in % des Antrieb-Nenndrehmoments
{
    LABEL		"Drehmomentwert Richtung ZU in % des Antrieb-Nenndrehmoments"; // [TXT_PZD_381];
    HELP		"Drehmomentwert Richtung ZU in % des Antrieb-Nenndrehmoments"; // [HLP_PZD_381];
    CLASS		DYNAMIC;
    TYPE		UNSIGNED_INTEGER (2)
    {
        MIN_VALUE	0;
        MAX_VALUE	127;
        DEFAULT_VALUE	0;
    }
    CONSTANT_UNIT		"%";
    HANDLING		READ;
}

COMMAND read_PZD_381
{
    SLOT		3;
    INDEX		209;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_381    // 2 Octets
        }
        REQUEST
        {
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  PZD_146
// 
VARIABLE PZD_146	// Drehmomentert Richtung AUF in Nm
{
    LABEL		"Drehmomentert Richtung AUF in Nm"; // [TXT_PZD_146];
    HELP		"Drehmomentert Richtung AUF in Nm"; // [HLP_PZD_146];
    CLASS		DYNAMIC;
    TYPE		UNSIGNED_INTEGER (4)
    {
        DEFAULT_VALUE	0;
    }
    CONSTANT_UNIT		"Nm";
    HANDLING		READ;
}

COMMAND read_PZD_146
{
    SLOT		1;
    INDEX		174;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_146    // 4 Octets
        }
        REQUEST
        {
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  PZD_3
// 
VARIABLE PZD_3	// Drehmomentwert Richtung ZU in Nm
{
    LABEL		"Drehmomentwert Richtung ZU in Nm"; // [TXT_PZD_3];
    HELP		"Drehmomentwert Richtung ZU in Nm"; // [HLP_PZD_3];
    CLASS		DYNAMIC;
    TYPE		UNSIGNED_INTEGER (4)
    {
        DEFAULT_VALUE	0;
    }
    CONSTANT_UNIT		"Nm";
    HANDLING		READ;
}

COMMAND read_PZD_3
{
    SLOT		0;
    INDEX		131;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_3    // 4 Octets
        }
        REQUEST
        {
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  PZD_378
// 
VARIABLE PZD_378	// Drehmomentwert Richtung AUF in ft-lb
{
    LABEL		"Drehmomentwert Richtung AUF in ft-lb"; // [TXT_PZD_378];
    HELP		"Drehmomentwert Richtung AUF in ft-lb"; // [HLP_PZD_378];
    CLASS		DYNAMIC;
    TYPE		UNSIGNED_INTEGER (2)
    {
        DEFAULT_VALUE	0;
    }
    CONSTANT_UNIT		"ft-lb";
    HANDLING		READ;
}

COMMAND read_PZD_378
{
    SLOT		3;
    INDEX		206;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_378    // 2 Octets
        }
        REQUEST
        {
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  PZD_379
// 
VARIABLE PZD_379	// Drehmomentwert Richtung ZU in ft-lb
{
    LABEL		"Drehmomentwert Richtung ZU in ft-lb"; // [TXT_PZD_379];
    HELP		"Drehmomentwert Richtung ZU in ft-lb"; // [HLP_PZD_379];
    CLASS		DYNAMIC;
    TYPE		UNSIGNED_INTEGER (2)
    {
        DEFAULT_VALUE	0;
    }
    CONSTANT_UNIT		"ft-lb";
    HANDLING		READ;
}

COMMAND read_PZD_379
{
    SLOT		3;
    INDEX		207;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_379    // 2 Octets
        }
        REQUEST
        {
        }
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining IMAGE  img_RC_status
 */
IMAGE img_RC_status
{
	PATH	IF (var_RC_active)
			{
				"RC_lediogn.gif";
			}
			ELSE
			{
				"RC_lediord.gif";
			}
	LINK	method_RC_toggle_enable;
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining IMAGE  img_RC_CLOSE_symbol
 */
IMAGE img_RC_CLOSE_symbol
{
	PATH  	IF (PZD_370)
			{
				"RC_symbols_close_hi.gif";
			} 
			ELSE 
			{
				"RC_symbols_close.gif";
			} 
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining IMAGE  img_RC_Tc_symbol
 */
IMAGE img_RC_Tc_symbol
{
	PATH  	IF (PZD_269)
			{
				"RC_symbols_Tc_hi.gif";
			} 
			ELSE 
			{
				"RC_symbols_Tc.gif";
			} 
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining IMAGE  img_RC_M_symbol
 */
IMAGE img_RC_M_symbol
{
	PATH  	IF (PZD_270)
			{
				"RC_symbols_M_hi.gif";
			} 
			ELSE 
			{
				"RC_symbols_M.gif";
			} 
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining IMAGE  img_RC_To_symbol
 */
IMAGE img_RC_To_symbol
{
	PATH  	IF (PZD_268)
			{
				"RC_symbols_To_hi.gif";
			} 
			ELSE 
			{
				"RC_symbols_To.gif";
			} 
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining IMAGE  img_RC_OPEN_symbol
 */
IMAGE img_RC_OPEN_symbol
{
	PATH  	IF (PZD_369)
			{
				"RC_symbols_open_hi.gif";
			} 
			ELSE 
			{
				"RC_symbols_open.gif";
			} 
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining IMAGE  img_RC_switch
 */
IMAGE img_RC_switch
{
	PATH  	SELECT (var_RC_switch) 
			{
				CASE 1: "RC_switch_local.gif";
				CASE 2: "RC_switch_off.gif";
				CASE 3: "RC_switch_remote.gif";
				DEFAULT: "RC_switch_off.gif";
			}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining METHOD	method_RC_toggle_enable
 */
METHOD	method_RC_toggle_enable
{
    LABEL		"Activate remote control|de|Ansteuerung bernehmen"; //[TXT_TST_314];
	ACCESS		ONLINE;
    DEFINITION
    {
		char sz[255];
		int rc;

		/* write activation order */
		/* in case of DPV0_Data-Exchange Drive will return negative Response, error code 6 (Access.access denied) */
		IGNORE_COMM_ERROR();
		IGNORE_COMM_STATUS(0xFFFFFFFF);
		IGNORE_RESPONSE_CODE(0xFFFFFFFF);
		var_RC_on= !var_RC_active;
		var_RC_on_device= var_RC_on;
		WriteCommand(write_RC_on);

		/* check success and availability */
		ReadCommand (read_var_RC_onoff);
		if (var_RC_on != var_RC_onoff)
		{
			if (!var_RC_onoff)
			{
				/* Annotation  */
				rc= get_dictionary_string(TXT_TST_1248, sz, 255);
			}
			else
			{
				/* Annotation  */
				rc= get_dictionary_string(TXT_TST_1249, sz, 255);
			}
			/* test showed: rc always "-1" --> to ignore so far */
			ACKNOWLEDGE (sz);
			return;
		}
		if ( var_RC_on && (var_RC_switch != 3) )
		{
			/* Annotation  */
			rc= get_dictionary_string(TXT_TST_1250, sz, 255);
			/* test showed: rc always "-1" --> to ignore so far */
			ACKNOWLEDGE (sz);
		}
		/* Save refreshed data */
        save_values();
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining METHOD	method_RC_clear_enable
 */
METHOD	method_RC_clear_enable
{
    LABEL		[TXT_TST_1251];
	ACCESS		ONLINE;
    DEFINITION
    {
		var_RC_on= 0;
		WriteCommand(write_RC_on);
		/* Save refreshed data */
        save_values();
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining METHOD  method_RC_move_open
 */
METHOD	method_RC_move_open
{
    LABEL		[TXT_TYP_64];
	ACCESS		ONLINE;
    DEFINITION
    {
		char sz[255];
		if (var_RC_active)
		{ 
			var_RC_cmd= RC_CMD_CODE_OPEN;
			WriteCommand(write_RC_cmd);
		}
		else
		{
			sz= get_dictionary_string(TXT_TST_1252, sz, 255);
			ACKNOWLEDGE (sz);  
		}		
		/* Save refreshed data */
        save_values();
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining METHOD  method_RC_move_close
 */
METHOD	method_RC_move_close
{
    LABEL		[TXT_TYP_360];
	ACCESS		ONLINE;
    DEFINITION
    {
		char sz[255];
		if (var_RC_active)
		{ 
			var_RC_cmd= RC_CMD_CODE_CLOSE;
			WriteCommand(write_RC_cmd);
		}
		else
		{
			sz= get_dictionary_string(TXT_TST_1252, sz, 255);
			ACKNOWLEDGE (sz);  
		}		
		/* Save refreshed data */
        save_values();
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining METHOD  method_RC_move_stop
 */
METHOD	method_RC_move_stop
{
    LABEL		[TXT_TYP_255];
	ACCESS		ONLINE;
    DEFINITION
    {
		char sz[255];
		if (var_RC_active)
		{ 
			var_RC_cmd= RC_CMD_CODE_STOP;
			WriteCommand(write_RC_cmd);
		}
		else
		{
			sz= get_dictionary_string(TXT_TST_1252, sz, 255);
			ACKNOWLEDGE (sz);  
		}		
		/* Save refreshed data */
        save_values();
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining METHOD  method_RC_cmd_reset
 */
METHOD	method_RC_cmd_reset
{
    LABEL		[TXT_TYP_241];
	ACCESS		ONLINE;
    DEFINITION
    {
		char sz[255];
		if (var_RC_active)
		{ 
			var_RC_cmd= RC_CMD_CODE_RESET;
			WriteCommand(write_RC_cmd);
		}
		else
		{
			sz= get_dictionary_string(TXT_TST_1252, sz, 255);
			ACKNOWLEDGE (sz);  
		}		
		/* Save refreshed data */
        save_values();
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining METHOD  method_RC_status_refresh
 */
METHOD	method_RC_status_refresh
{
    LABEL		"method_RC_status_refresh";	/* [TXT_method_RC_refresh_display] */
	ACCESS		ONLINE;
    DEFINITION
    {
		float val;
		/* refresh var_RC_active status */
		if ((var_RC_switch.VARIABLE_STATUS == VARIABLE_STATUS_LOADED) && (var_RC_switch == 3) && var_RC_onoff)
		{
			var_RC_active= 1;
		}
		else
		{
			var_RC_active= 0;
		}
		/* Save refreshed data */
        save_values();
    }
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining METHOD  method_RC_torque_refresh
 */ 
METHOD method_RC_torque_refresh
{
    LABEL       "method_RC_torque_refresh";
    DEFINITION
    {   
		// refresh torque value
		switch (var_RC_torque_unit) 
		{
		    default: 
			case 1: 
				if (PZD_381 != 0) { var_RC_torque= PZD_381; } else { var_RC_torque= PZD_380; }
				break;
			case 2:
				if (PZD_3 != 0) { var_RC_torque= PZD_3; } else { var_RC_torque= PZD_146; }
				break;
			case 3:
				if (PZD_379 != 0) { var_RC_torque= PZD_379; } else { var_RC_torque= PZD_378; }
				break;
		}
		var_RC_torque.VARIABLE_STATUS= VARIABLE_STATUS_LOADED;
		/* Save refreshed data */
        save_values();
    }
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining MENU  menu_RC_main -- root menu for remote control
 */
MENU menu_RC_main
{
    LABEL		"Remote control|de|Fernbedienung"; //[TXT_TST_316];
	STYLE		WINDOW;
	ACCESS		ONLINE;
    ITEMS
    {
		menu_RC_enable_group
		, ROWBREAK
		, menu_RC_status_group
		, ROWBREAK
		, menu_RC_control_group
		, ROWBREAK
		, menu_RC_switch_group
    }
	POST_EDIT_ACTIONS { method_RC_clear_enable } /* doesn't work in PDM6 */
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining MENU  menu_RC_enable_group
 */
MENU menu_RC_enable_group
{
    LABEL		"";
	STYLE		GROUP;
	ACCESS		ONLINE;
    ITEMS
    {
		/* used device VARIABLES to get refreshed */
		var_RC_onoff (HIDDEN)
		img_RC_status
	}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining MENU  menu_RC_status_group
 */
MENU menu_RC_status_group
{
    LABEL		"";
	STYLE		GROUP;
	ACCESS		ONLINE;
    ITEMS
    {
		/* device variable for refreshing display status */
		PZD_268 (HIDDEN)
		, PZD_269 (HIDDEN)
		, PZD_270 (HIDDEN)
		, PZD_369 (HIDDEN)
		, PZD_370 (HIDDEN)
		, img_RC_CLOSE_symbol (INLINE)
		, COLUMNBREAK
		, img_RC_Tc_symbol (INLINE)
		, COLUMNBREAK
		, img_RC_M_symbol (INLINE)
		, COLUMNBREAK
		, img_RC_To_symbol (INLINE)
		, COLUMNBREAK
		, img_RC_OPEN_symbol (INLINE)
	}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining MENU  menu_RC_control_group
 */
MENU menu_DBG
{
    LABEL		"DEBUG";
	STYLE		GROUP;
	ACCESS		ONLINE;
    ITEMS
    {
		var_RC_torque_unit, ROWBREAK
		, PZD_380, COLUMNBREAK, PZD_381, ROWBREAK
		, PZD_146, COLUMNBREAK, PZD_3, ROWBREAK
		, PZD_378, COLUMNBREAK, PZD_379, ROWBREAK
	}
}

 
MENU menu_RC_control_group
{
    LABEL		"";
	STYLE		GROUP;
	ACCESS		ONLINE;
    ITEMS
    {
		IF (var_RC_torque_unit == 1) { PZD_380 (HIDDEN), PZD_381 (HIDDEN) }
		IF (var_RC_torque_unit == 2) { PZD_146 (HIDDEN), PZD_3 (HIDDEN) }
		IF (var_RC_torque_unit == 3) { PZD_378 (HIDDEN), PZD_379 (HIDDEN) }
		/// menu_DBG, 
		var_RC_position (READ_ONLY)
		, COLUMNBREAK
		, method_RC_move_open
		, ROWBREAK
		, var_RC_torque_unit
		, COLUMNBREAK
		, method_RC_move_stop
		, ROWBREAK
		, var_RC_torque (READ_ONLY)
		, COLUMNBREAK
		, method_RC_move_close
		, method_RC_cmd_reset
	}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * Defining MENU  menu_RC_switch_group
 */
MENU menu_RC_switch_group
{
    LABEL		"";
	STYLE		GROUP;
	ACCESS		ONLINE;
    ITEMS
    {
		/* device variable for refreshing display status */
		var_RC_switch (HIDDEN)
		, [TXT_TYP_71]
		, COLUMNBREAK
		, img_RC_switch (INLINE)
		, COLUMNBREAK
		, [TXT_TYP_70]
	}
}

/* ---------------------------------------------------------------------------------------------------------------------
 * End of file
 */
